#ifndef _MCW_MACHDEP_
#define _MCW_MACHDEP_

#ifdef SPARKY
#undef _POSIX_SOURCE
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

/*----------------------------------------------------------------------------
  Flags that can be used to work around bugs on some systems
  (you could also use the -Dname command line switch in the Makefile
   definition of the CC commands to enable these options):

    DONT_USE_METER  = if #define-d, won't show progress meter during
                       brick write operations

    FIX_SCALE_VALUE_PROBLEM = if #defined-d, will work around a bug
                               in Solaris Motif where the threshold scale
                               value is not displayed
                               [Doesn't seem to be needed anymore]

    FIX_SCALE_SIZE_PROBLEM = if #define-d, will work around a bug in
                              some versions of Motif where the
                              threshold scale resizes itself whenever
                              the pbar is touched

    SCANDIR_WANT_CONST = if #define-d, says that the "scandir" library
                          routine wants "const" arguments -- setting this
                          flag will avoid some stupid compiler warnings

    DONT_USE_SCANDIR   = if #define-d, the Unix routine scandir won't
                          be used.  This seems to help on Solaris, and
                          doesn't hurt on other systems, so it is now
                          the default.

    DONT_INSTALL_ICONS = if #define-d, won't try to install icons for the
                          various windows (Sun's OpenWindows complains when
                          I do this, and I don't know why, and I don't care)

    DONT_CHECK_FOR_MWM = if this is set, then the program won't bother
                          to check for the Motif Window Manager (MWM).
                          This can be useful on Linux systems using FVWM,
                          which won't be detected as MWM, but can be set up
                          to act like it with regards to decorations, etc.

    BOXUP_SCALE = if this is set, then the slider for the functional
                  threshold will have a "box" (Frame widget) drawn
                  around it.  Some people think this looks nicer, some don't.

    NO_FRIVOLITIES = if this is set, then the hidden "fun" parts of
                     AFNI are disabled.  What these are is a secret.

    DONT_USE_HINTS = if this is set, then the popup hints won't be
                     compiled into AFNI.

    NEED_XSETLOCALE = if this is set, then the routine _Xsetlocale
                      must be provided (needed for some Linux systems).

    DONT_UNROLL_FFTS = if this is set, then the unrolled FFT routines
                       (for lengths 32, 64, 128, 256) will NOT be used --
                       they are generally faster, but may have trouble
                       compiling on some systems [see file csfft.c].
                       The program fftest.c can be used to test the
                       speed of FFTs.

    DONT_USE_MCW_MALLOC = if this is set, then the malloc wrappers
                          defined in mcw_malloc.[ch] will not be
                          be used.  (These functions provide some
                          ability to track and debug the use of
                          malloc-ed memory space.)

    SOLARIS_DIRENT_PATCH = if this is set, then a patch for the
                           difference between Sun's "dirent" functions
                           and everbody else's is used in the file
                           mcw_glob.c -- this seems to be necessary
                           on Solaris 2.6 systems in order to get the
                           AFNI file reading software to work.
                           (This patch was supplied by Christoph Losert
                           of the Institut fuer Radiologische Diagnostik,
                           Munchen, and has not been tested at MCW due to
                           the absence of any Sun workstations to play with.)

    SOLARIS_DIRENT_ZERO  = if this is defined, the patch described above
                           is modified slightly -- see mcw_glob.c for
                           the gory details.  The patch above seems to
                           be needed for Solaris 2.5.x and this modification
                           is also needed for Solaris 2.6.x.
                              [God, I hope this is the end of this ]
                              [Solaris nightmare -- I'm sick of it!]

  Exactly one of the following flags must be set for AFNI plugins
  to work:

    DYNAMIC_LOADING_VIA_DL = if this is set, then loading of dynamic
                             libraries is accomplished using the "dl"
                             routines, such as "dlopen".  This is true
                             for all Unixes that I know of, except for
                             HP-UX.

    DYNAMIC_LOADING_VIA_SHL = if this is set, then loading of dynamic
                              libraries is accomplished using the "shl"
                              routines, such as "shl_load".  This is
                              only used on HP-UX, as far as I know.

  Flags that MUST be set appropriately for each system:

    THD_MMAP_FLAG = value to set when using "mmap" to map a file to memory
                     (see man mmap to figure this one out)

    THD_MKDIR_MODE = flag to set creation mode for directories that might
                      be created during dataset output (you can probably
                      copy this from any other machine)

    NO_RINT = #define this if the system you are on doesn't have the
              "rint" (round-to-integer) math function -- I don't know
              of any system without this function, but you never can tell.

    DONT_USE_STRPTIME = #define this if your system doesn't have the
                        C function strptime()

  Some systems need extra header files included.  Some system header
  files don't give a prototype for alphasort.  This is a place to fix
  these things up.
-------------------------------------------------------------------------*/

#define DONT_USE_SCANDIR
#define DONT_UNROLL_FFTS  /* off by default */

/*** HP-UX ***/

#ifdef HP
# include <dirent.h>
# define THD_MMAP_FLAG  (MAP_FILE | MAP_VARIABLE | MAP_SHARED)
# define THD_MKDIR_MODE (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
# define SCANDIR_WANTS_CONST
# define BOXUP_SCALE
# define FIX_SCALE_SIZE_PROBLEM
# define DYNAMIC_LOADING_VIA_SHL
# undef  DONT_UNROLL_FFTS    /* FFTs are faster, but csfft.c compiles slowly */
#endif

/*** SGI IRIX ***/

#ifdef SGI
# include <dirent.h>
# define THD_MMAP_FLAG  MAP_SHARED
# define THD_MKDIR_MODE (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
# define BOXUP_SCALE
# define DYNAMIC_LOADING_VIA_DL
# define FIX_SCALE_SIZE_PROBLEM
# ifndef DONT_USE_SCANDIR
    extern int alphasort(struct dirent **, struct dirent **) ;
# endif
# undef  DONT_UNROLL_FFTS         /* helps some */
#endif

/*** SunOS or Solaris ***/

#ifdef SPARKY
# include <sys/dirent.h>
# define THD_MMAP_FLAG  (MAP_SHARED | MAP_NORESERVE)
# define THD_MKDIR_MODE (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
# define DONT_INSTALL_ICONS
# define NO_FRIVOLITIES
# define FIX_SCALE_SIZE_PROBLEM
# ifndef DONT_USE_SCANDIR
    extern int alphasort(struct dirent **, struct dirent **) ;
# endif

extern double strtod() ;
extern long   strtol() ;

/**
#ifndef S_IFREG
#  define S_IFREG _IFREG
# endif
# ifndef S_IFDIR
#  define S_IFDIR _IFDIR
# endif
**/
#endif

#if defined(SOLARIS) || defined(SUN)
# include <sys/types.h>
# ifndef DONT_USE_SCANDIR
#   include <sys/dir.h>
    extern int alphasort(struct dirent **, struct dirent **) ;
# endif
# define THD_MMAP_FLAG  (MAP_SHARED | MAP_NORESERVE)
# define THD_MKDIR_MODE (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
# define dirent direct
# define FIX_SCALE_SIZE_PROBLEM
# define DONT_INSTALL_ICONS
# define DYNAMIC_LOADING_VIA_DL
#endif

/*** IBM RS6000 courtesy Doug Morris of UIUC ***/

#ifdef RS6000
# include <dirent.h>
# define THD_MMAP_FLAG  MAP_SHARED
# define THD_MKDIR_MODE (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
# define FIX_SCALE_SIZE_PROBLEM
# ifndef DONT_USE_SCANDIR
    extern int alphasort(struct dirent **, struct dirent **) ;
# endif
#endif

/*** Linux 1.2.x ***/

#ifdef LINUX2
# define NEED_XSETLOCALE
# ifndef LINUX
#   define LINUX
# endif
#endif

#ifdef LINUX
# include <dirent.h>
# define THD_MMAP_FLAG  MAP_SHARED
# define THD_MKDIR_MODE (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
# define SCANDIR_WANTS_CONST
# define FIX_SCALE_SIZE_PROBLEM   /* Motif 2.0 bug? */
/* # define MMAP_THRESHOLD -1 */       /* no mmap-ing */
# define DONT_CHECK_FOR_MWM       /* assume Motif WM functionality is present */
# define BOXUP_SCALE              /* looks nicer */
# define DYNAMIC_LOADING_VIA_DL
# undef  DONT_UNROLL_FFTS         /* helps a lot */
#endif

/************************************************************************
   Do NOT change anything below this line (unless your name is Cox)!
*************************************************************************/

#if defined(DYNAMIC_LOADING_VIA_DL) || defined(DYNAMIC_LOADING_VIA_SHL)
#  define ALLOW_PLUGINS
#else
#  define DONT_ALLOW_PLUGINS
#endif

#endif /* _MCW_MACHDEP_ */
