function dtautoseg(dtensorstem, maskstem, clusters, outdir)
version = '$Id: dtautoseg.m,v 1.2 2003/07/08 04:11:36 dtuch Exp $';

fprintf('dtautoseg\n');

[rows cols frames slice0 slices endian bext] = fmri_bfiledim(maskstem);
if (isempty(rows))
  error(sprintf('ERROR: loading %s\n',maskstem));
  exit;
end 

% load data
tic;
fprintf('Loading mask data ... ');
mask = fmri_ldbvolume(maskstem);
if (isempty(mask))
  fprintf('ERROR: loading %s\n',maskstem);
  exit;
end
fprintf('done (%g)\n',toc);


% load data
tic;
fprintf('Loading tensor data ... ');
dtensor = fmri_ldbvolume(dtensorstem);
if (isempty(dtensor))
  fprintf('ERROR: loading %s\n', dtensorstem);
  exit;
end
fprintf('done (%g)\n',toc);


fprintf('Extracting mask coordinates ... ');
tic;
midx = find(mask);
[slice row col] = ind2sub(size(mask), midx);
x = [slice row col];
d = zeros(length(x), 9);
for i = 1:length(x)
  d(i,:) = dtensor(x(i,1),x(i,2),x(i,3),:);
end
fprintf('done (%g)\n',toc);

fprintf('Computing local linear embedding ... ');
v = lle(x, d, clusters);
fprintf('done (%g)\n',toc);

fprintf('Calculating initial covariance matrices ... ');
tic;
nm = sqrt(diag(v*v'));
[num bins] = hist(nm,length(v));
num = num/sum(num);
cdf = cumsum(num);
[null j] = min(abs(cdf-.95));
vouter = v(nm>bins(j),:);

[ev, el] = eig(cov(vouter));
el = diag(el);

% initialize cov matrices
sigma = zeros(clusters,clusters,clusters);
for i = 1:clusters
  eltmp = el;
  eltmp(setdiff(1:clusters,i)) = el(i)/2;
  sigma(:,:,i) = ev*diag(eltmp)*ev';
end
fprintf('done (%g)\n',toc);

fprintf('Calculating EM algorithm on LLE ... ');
tic;
[p mu sigma soft] = em(v, clusters, sigma, 1);
fprintf('done (%g)\n',toc);

fprintf('Constructing soft segmentation images ... ');
tic;
for cluster = 1:clusters
  softimage(cluster,:,:,:) = zeros(slices, rows, cols);
  for i = 1:length(x)
    softimage(cluster,x(i,1),x(i,2),x(i,3)) = Z(i,cluster);
  end  
end
fprintf('done (%g)\n',toc);

fprintf('Constructing hard segmentation images ... ');
tic;
hardimage = zeros(clusters,slices,rows,cols);
[null j] = max(soft,[],2);
for i = 1:length(x)
  hardimage(j(i),x(i,1),x(i,2),x(i,3)) = 1;
end
fprintf('done (%g)\n',toc);

for i = 0:(clusters-1)
  fname = sprintf('soft-%03d', i);
  fprintf(['Saving ' fname '\n']);
  softstem = [outdir '/' fname];
  fmri_svbvolume(squeeze(softimage(i+1,:,:,:)), softstem);
end

for i = 0:(clusters-1)
  fname = sprintf('hard-%03d', i);
  fprintf(['Saving ' fname '\n']);
  hardstem = [outdir '/' fname];
  fmri_svbvolume(squeeze(hardimage(i+1,:,:,:)), hardstem);
end

fprintf('Saving cluster fractions\n');
dlmwrite([outdir '/p.txt'], p, ' ')

fprintf('dtautoseg done\n');
exit;















