% dtrotate
%
% rotate tensor file with specified rotation matrix
%
% input: instem, matrixfile
%
% output: dtensor

function dtrotate(instem, matfile, outdir)
version = '$Id: dtrotate.m,v 1.4 2003/07/28 13:41:59 dtuch Exp $';

fprintf('dtrotate\n');

try
  [rows cols frames slice0 slices endian bext] = fmri_bfiledim(instem);
catch
  error(sprintf('ERROR: loading %s\n',instem));
  exit;
end

R = load(matfile, '-ascii');
R = affine2rot(R);
invR=pinv(R);

% memory
tic;
fprintf('Allocating memory ... ');
dtensor = zeros(slices, rows, cols, 9);
fprintf('done (%g)\n',toc);

% load data
tic;
fprintf('Loading data ... ');
try
  dtensor = fmri_ldbvolume(instem);
catch
  fprintf('ERROR: loading %s\n',instem);
  exit;
end
fprintf('done (%g)\n',toc);

fprintf('Rotating tensors ... \n');
for slice = 1:slices
  fprintf('%d ', slice - 1);
  for row = 1:rows
    for col = 1:cols

      D = reshape(squeeze(dtensor(slice, row, col, :)), [3 3]);
      
      if ( all(D(:)) )
	D = R*D*invR;
	dtensor(slice,row,col,:) = D(:);
      end      
    end
  end  
end

fprintf('\n');

fprintf('Saving dtensor\n');
outstem = sprintf('%s/dtensor',outdir);
fmri_svbvolume(squeeze(reshape(dtensor, [slices rows cols 9])), outstem);

fprintf('dtrotate done\n');
exit;

