%
% tensor2odf
%
% convert tensor volume to ODF volume
%
% input:
% instem, outdir, dtime
%
% output:
% odf, gfa

function tensor2odf(instem, outdir, dtime)
version = '$Id: tensor2odf.m,v 1.1 2003/09/12 17:43:14 dtuch Exp $';
 
fprintf('tensor2odf\n');

dtime = str2num(dtime);

[rows cols frames slice0 slices endian bext] = fmri_bfiledim(instem);
if(isempty(rows))
  error(sprintf('ERROR: loading %s\n',instem));
  exit;
end 

% load data
tic;
fprintf('Loading data ... ');
dtensor = fmri_ldbvolume(instem);
if (isempty(dtensor))
  fprintf('ERROR: loading %s\n',instem);
  exit;
end
fprintf('done (%g)\n',toc);
dtensor = reshape(dtensor, [slices rows cols 3 3]);

fprintf('Image dimensions:\n');
fprintf('  slices/rows/cols = %d/%d/%d\n', slices, rows, cols);

recondirfilename = 'dodeca5-755dir.txt';
recondirs = load(recondirfilename);

tic;
fprintf('Constructing mask ...');
mask = any(dtensor, 4);
fprintf(' done (%g)\n',toc);

tic;
fprintf('Allocating memory ...');
gfamap = zeros(slices, rows, cols);
fprintf(' done (%g)\n',toc);

tic;
fprintf('Computing ODFs ...\n');
for slice = 1:slices
  odfmap = zeros(rows, cols, length(recondirs));
  fprintf('%d ', slice - 1);
  for row = 1:rows
    for col = 1:cols
      
      if (mask(slice, row, col))
	dinv = pinv(squeeze(dtensor(slice, row, col, :, :)));
	odf = exp(diag(recondirs*dinv*recondirs')/(4*dtime));
	odf = odf/sum(odf);

	odfmap(row, col, :) = odf;
	gfamap(slice,row,col) = gfa(odf);
      end
      
    end
  end  
  filename = sprintf('%s/odf_%03d.bfloat', outdir, slice-1);
  fmri_svbfile(odfmap, filename, endian);

end
fprintf(' done (%g)\n',toc);
fprintf('\n');

fprintf('Saving gfa\n');
outstem = sprintf('%s/gfa', outdir);
fmri_svbvolume(gfamap, outstem);

fprintf('tensor2odf done\n');
exit;

