% mlr
%
% multiple linear regression
%
% input: tablefname, instemlist, outdir
%
% output: rsquared, p, beta

function mlr(regressorfile, instemlist, outdir)
version = '$Id: mlr.m,v 1.7 2003/08/01 19:13:28 dtuch Exp $';
 
fprintf('mlr\n');
frpintf('vesion: %s\n\n', version);

instems = textread(instemlist, '%q');

n = size(instems,1);
fprintf('Group size: n=%d\n', n);
fname = char(instems(1));
try
  [rows cols frames slice0 slices endian bext] = ...
      fmri_bfiledim(fname);
catch
  fprintf('ERROR: loading %s\n', fname);
  exit;
end

  fprintf ('Image dimensions:\n');
fprintf('  slices/rows/cols = %d/%d/%d\n', slices, rows, cols);

% load regressors
try
  X = load(regressorfile);
catch
  fprintf('ERROR: unable to read regressor list');
  exit;
end


X = [ones(size(X,1),1) X];
if (size(X,1)~=n)
  fprintf('ERROR: number of rows in regressor list does not equal number of subjects');
  exit;
end


% memory
tic;
fprintf('Allocating memory ... ');
data = zeros(n, slices, rows, cols);
rsquared = zeros(slices,rows,cols);
p = zeros(slices,rows,cols);
beta = zeros(slices,rows,cols,size(X,2));
fprintf('done (%g)\n',toc);

% load data
tic;
fprintf('Loading data ... \n');
for member = 1:n
  fname = char(instems(member));
  fprintf('  %s\n', fname);
  try
    a = fmri_ldbvolume(fname);
  catch
    fprintf('ERROR: loading %s\n',fname);
    exit;
  end
  data(member,:,:,:) = a;
end
fprintf('done (%g)\n',toc);

mask = squeeze(all(data,1));

tic;
fprintf('Computing regression map ... \n');
for slice = 1:slices
fprintf('%g ', slice-1);
  for row = 1:rows
    for col = 1:cols
      if mask(slice,row,col)
	[b,bint,r,rint,stats] = ...
	    regress(squeeze(data(:,slice,row,col)),X);	
	rsquared(slice,row,col) = stats(1);
	p(slice,row,col) = -log10(stats(3));
	beta(slice,row,col,:) = b;
      end
    end
  end
end
fprintf('\ndone (%g)\n',toc);

fprintf('Saving rsquared\n');
outstem = [outdir '/rsquared'];
fmri_svbvolume(rsquared,outstem);

fprintf('Saving p\n');
outstem = [outdir '/p'];
fmri_svbvolume(p,outstem);

fprintf('Saving beta\n');
outstem = [outdir '/beta'];
fmri_svbvolume(squeeze(beta),outstem);

fprintf('mlr done\n');
exit;



