function [p, mu, sigma, Z] = em(x, k, sigma, verbose)

[n D] = size(x);      % number of observations (n) and dimension (D)
p = ones(1,k)/k;      % mixing proportions
mu = zeros(D,k);      % means
maxiter = 200;        % number of iterations

clear Z;
% run EM 
fprintf('\n');
if (verbose)
  fprintf('Running EM ...\n');
end

iter = 1;
[perr muerr sigmaerr] = deal(1,1,1);
minerr = 10^-10;

while ((iter < maxiter) & ...
       (perr > minerr) | ...
       (muerr > minerr) | ...
       (sigmaerr > minerr)) 						  

  % E-step
  for i = 1:k
    Z(:,i) = p(i)*det(sigma(:,:,i))^(-0.5)*exp(-0.5*sum((x'-repmat(mu(:,i),1,n))'*inv(sigma(:,:,i)).*(x'-repmat(mu(:,i),1,n))',2));
  end
  Z = Z./repmat(sum(Z,2),1,k);

  % M-step:
  for i = 1:k
    [prevp prevmu prevsigma] = deal(p, mu, sigma);
    mu(:,i) = (x'*Z(:,i))./sum(Z(:,i));
    sigma(:,:,i) = real((x'-repmat(mu(:,i),1,n))*(repmat(Z(:,i),1,D).*(x'-repmat(mu(:,i),1,n))')./sum(Z(:,i)));
    p(i) = mean(Z(:,i));    
  end
  
  % calc errs for convergence criterion
  sigmaerr = norm(prevsigma(:)-sigma(:));
  perr = norm(prevp(:)-p(:));
  muerr = norm(prevmu(:)-mu(:));
  if (verbose)
    fprintf('iteration: %g perr: %f muerr: %f sigmaerr: %f\n', ...
	    iter, log10(perr), log10(sigmaerr), log10(muerr));
  end
  iter = iter + 1;
end

