/*******************************************
 * I2bshort-anat.c
 * 
 * Converts GE files to bshort files.  Converts
 *  I.xxx to I_xxx.bshort and I_xxx.hdr
 * where xxx goes from firstslice to firstslices+nslices
 *
 * USAGE: I2bshort-anat firstslice nslices rows cols 
 * 
 * Douglas N. Greve
 * April 3, 1999
 *******************************************/
#include<stdio.h>
#include<stdlib.h>
#include<strings.h>

long int FileSizeBytes(FILE *pF);

char infile[128], outbase[128];
char bshortfile[128], hdrfile[128];
char zstr[10];
FILE *infp,*outfp;
char *c;
int i,j,k;
int offset;
int rows,cols,slices;
int firstslice, nslices;
 
int main(int argc, char **argv)
{
  if(argc != 5){
    fprintf(stderr,"USAGE: I2bshort-anat firstslice nslices rows cols \n");
    fprintf(stderr,"Converts GE files to bshort files.  The input files \n"
            "are I.xxx where xxx goes from firstslice to firstslices+nslices\n"
            "The output files are I_xxx.bshort and I_xxx.hdr\n");

    exit(1);
  }
  sscanf(argv[1],"%d",&firstslice);
  sscanf(argv[2],"%d",&nslices);
  sscanf(argv[3],"%d",&rows);
  sscanf(argv[4],"%d",&cols);
  

  for(i=firstslice; i< firstslice+nslices; i++){

    if(i<10)       sprintf(zstr,"00%d",i);
    else if(i<100) sprintf(zstr,"0%2d",i);
    else           sprintf(zstr,"%3d",i);

    sprintf(infile,"I.%s",zstr);
    fprintf(stderr,"Converting %s\n",infile);

    infp = fopen(infile,"rb");
    if(infp==NULL){
      fprintf(stderr,"Error: cannot open %s\n",infile);
      exit(1);
    } 

    k = FileSizeBytes(infp);
    c = (char *) malloc(k*sizeof(char));
    fread(c, sizeof(char), k, infp);
    fclose(infp);

    offset = k - rows*cols*2;

    if(offset < 0){
      fprintf(stderr,"Error: offset (%d) exceeds file size (%d)\n",
	      offset,k);
      exit(1);
    } 
  
    sprintf(bshortfile,"I_%s.bshort",zstr);
    outfp = fopen(bshortfile,"wb");
    if(outfp==NULL){
      fprintf(stderr,"Error: cannot open outfile %s\n",bshortfile);
      exit(1);
    } 

    j = fwrite((void *)(&c[offset]),sizeof(char),k-offset,outfp);
    fclose(outfp);

    sprintf(hdrfile,"I_%s.hdr",zstr);
    outfp = fopen(hdrfile,"w");
    fprintf(outfp,"%d %d 1 0\n",rows,cols);
    fclose(outfp);  
  }

  return(0);
}

long int FileSizeBytes(FILE *pF)
{
    long int CurPos, EndPos;
    
    CurPos = ftell(pF);
    fseek(pF,0L,SEEK_END);
    EndPos = ftell(pF);
    fseek(pF,0L,CurPos);
    return(EndPos);
}
