/*******************************************
 * I2bshort-func.c
 * 
 * Converts GE files to bshort files.  Converts
 * I.fff.saaa to I_aaa.bshort and I_aaa.hdr
 * where fff goes from f1 to f1+nf and aaa goes
 * from a1 to a1+na.
 *
 * USAGE: I2bshort-func f1 nf a1 na
 * 
 * Douglas N. Greve
 * April 3, 1999
 *******************************************/
#include<stdio.h>
#include<stdlib.h>
#include<strings.h>

long int FileSizeBytes(FILE *pF);

char infile[128], outbase[128];
char bshortfile[128], hdrfile[128];
char fstr[10],astr[10];
FILE *infp,*outfp;
char *c;
int i,j,k;
int offset;
int rows,cols,slices;
int f1, nf,f, a1,na,a;
 
int main(int argc, char **argv)
{
  if(argc != 7){
    fprintf(stderr,"USAGE: I2bshort-func f1 nf a1 na rows cols \n");
    fprintf(stderr,"   f1 - number of first functional slice\n");
    fprintf(stderr,"   nf - number functional slices\n");
    fprintf(stderr,"   a1 - number of first anatomical slice\n");
    fprintf(stderr,"   na - number anatomical slices\n");
    fprintf(stderr,"   rows - number of rows in each functional slice\n");
    fprintf(stderr,"   cols - number of cols in each functional slice\n");
    fprintf(stderr,"   Input files are: I.fff.saa where fff goes from\n");
    fprintf(stderr,"     f1 to f1+nf and aa goes from a1 to a1+na\n");
    fprintf(stderr,"   Output files are: I_aaa.bshort\n");
    exit(1);
  }
  sscanf(argv[1],"%d",&f1);
  sscanf(argv[2],"%d",&nf);
  sscanf(argv[3],"%d",&a1);
  sscanf(argv[4],"%d",&na);
  sscanf(argv[5],"%d",&rows);
  sscanf(argv[6],"%d",&cols);
  

  for(a=a1; a < a1+na; a++){

    if(a<10)       sprintf(astr,"00%d",a);
    else if(a<100) sprintf(astr,"0%2d",a);
    else           sprintf(astr,"%3d",a);

    sprintf(bshortfile,"I_%s.bshort",astr);
    fprintf(stderr,"Creating %s\n",bshortfile);

    outfp = fopen(bshortfile,"wb");
    if(outfp==NULL){
      fprintf(stderr,"Error: cannot open outfile %s\n",bshortfile);
      exit(1);
    } 

    for(f=f1; f < f1+nf; f++){

      if(f<10)       sprintf(fstr,"00%d",f);
      else if(f<100) sprintf(fstr,"0%2d",f);
      else           sprintf(fstr,"%3d",f);

      sprintf(infile,"I.%s.s%d",fstr,a);

      infp = fopen(infile,"rb");
      if(infp==NULL){
	fprintf(stderr,"Error: cannot open %s\n",infile);
	exit(1);
      } 

      k = FileSizeBytes(infp);
      c = (char *) malloc(k*sizeof(char));
      fread(c, sizeof(char), k, infp);
      fclose(infp);

      offset = k - rows*cols*2;
      if(offset < 0){
	fprintf(stderr,"Error: offset (%d) exceeds file size (%d)\n",
		offset,k);
	exit(1);
      } 
  
      fwrite((void *)(&c[offset]),sizeof(char),k-offset,outfp);
    }
    fclose(outfp);

    sprintf(hdrfile,"I_%s.hdr",astr);
    outfp = fopen(hdrfile,"w");
    fprintf(outfp,"%d %d %d 0\n",rows,cols,nf);
    fclose(outfp);  
  }

  return(0);
}

long int FileSizeBytes(FILE *pF)
{
    long int CurPos, EndPos;
    
    CurPos = ftell(pF);
    fseek(pF,0L,SEEK_END);
    EndPos = ftell(pF);
    fseek(pF,0L,CurPos);
    return(EndPos);
}
