/**************************************************************
 Name:    btavg.c
 Purpose: computes the temporal average for a bshort or bfloat
          for a given roi.
 $Id$
 **************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <sys/stat.h>

#define  BSHORT 0
#define  BFLOAT 1

long int getfilesize(char *fname);

FILE *infp;
long i,j,k,nbytes,n0,n1,m0,m1,Nv;
long dof;
short *dbs;
float *dbf;
double sum, sum2;
double avg, std, var;
char  ext[6];
char  stem[256];
char  infile[256];
char  hdrfile[256];

int   fl,btype;
int   nRows,nCols,Ntp,Endian;
int   r,r1,r2,c,c1,c2;

float v,vmin,vmax;


int main(int argc, char **argv)
{
  if(argc != 6){
    fprintf(stderr,"USAGE: btavg bfile r1 c1 r2 c2\n");
    fprintf(stderr,"       Version: $Id$\n");
    exit(1);
  }

  sprintf(infile,"%s",argv[1]);
  sscanf(argv[2],"%d",&r1);
  sscanf(argv[3],"%d",&c1);
  sscanf(argv[4],"%d",&r2);
  sscanf(argv[5],"%d",&c2);

  
  /******* get stem and extension *************/
  fl = strlen(infile);
  sprintf(ext,"%s",&(infile[fl-6]));
  for(i=0;i<fl-6;i++) stem[i] = infile[i];
  
  if(!strcmp(ext,"bshort"))      btype = BSHORT;
  else if(!strcmp(ext,"bfloat")) btype = BFLOAT;  
  else{
    fprintf(stderr,"Extension %s not recognized\n",ext);
    exit(1);
  }

  sprintf(hdrfile,"%shdr",stem);
  infp = fopen(hdrfile,"rb");
  if(infp==NULL){
    fprintf(stderr,"Error: cannot open %s\n",hdrfile);
    exit(1);
  } 
  fscanf(infp,"%d %d %d %d",&nCols,&nRows,&Ntp,&Endian);
  fclose(infp);
  Nv = (long)nRows*nCols;
  /*printf("%d %d %d %d\n",nCols,nRows,Ntp,Endian);*/

  /******** open the input file ******/  
  infp = fopen(infile,"rb");
  if(infp==NULL){
    fprintf(stderr,"Error: cannot open %s\n",infile);
    exit(1);
  } 

  /******* get number of bytes in file ********/
  nbytes = getfilesize(infile);

  /****** Load the data in ***************/
  if(btype == BSHORT){
    k = nbytes/sizeof(short);
    dbs = (short *) malloc(k*sizeof(short));
    dbf = (float *) malloc(k*sizeof(float));
    i = fread(dbs, sizeof(short), k, infp);
    for(j=0;j<k;j++) dbf[j] = (float) dbs[j];
    free(dbs);
  }
  else{
    k = nbytes/sizeof(float);
    dbf = (float *) malloc(k*sizeof(float));
    i = fread(dbf, sizeof(float), k, infp);
  }
  fclose(infp);



  sum = 0;
  sum2 = 0.0;
  vmin = +10000000000000000000000.0;
  vmax = -10000000000000000000000.0;
  dof = 0;
  for(r=r1;r<=r2;r++){
    for(c=c1;c<=c2;c++){
      n1 = r*nCols + c;
      for(k=0;k<Ntp;k++){
        n0 = k*Nv + n1;
	v = dbf[n0];
	sum += v;
	sum2 += v*v;
	if(v > vmax) vmax = v;
	if(v < vmin) vmin = v;	
	dof ++;
      }
    }
  }
  

  avg = sum/dof;
  var = (sum2 - dof*avg*avg)/(dof-1);
  std = sqrt(var);
  /* printf("%ld %ld %g %g %g %g %g\n",n0,n1,sum,sumsqrd,avg,var,std);*/
  printf("%10.4f %10.4f %10.4f %10.4f %7ld\n",avg,std,vmin,vmax,dof);
  

  return(0);
}

long int getfilesize(char *fname)
{
  struct stat filestat;
  int e;
  
  e = stat(fname, &filestat);
  if(e != 0){
    fprintf(stderr,"ERROR: Input File %s\n",fname);
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }

  return((long int)filestat.st_size);
}

