/****************************************************
  Name:    synthbfile.c
  USAGE:   synthbfile outstem format fs ns mean namp
  Purpose: simple tool to generate bfiles (either
           bshort or bfloat).

  Author: Douglas Greve, MGH-NMR
  Date:   May 5, 1999
  $Id$
*****************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <ctype.h>
#define ISOPTION(c)  ((c) == '-')

static int get_option(int argc, char *argv[]);
static void print_version(void);
static void print_usage(void);
static void usage_exit(void);
static void print_help(void);
int check_slice_plane(char *SlicePlane);

char OutStem[256];
char Subject[256];
char Volume[256];
char SubjectsDir[256];
char CorDir[256];
char SlicePlane[16];
int  SlicePlaneId = -1;

int  ResBetPlanes = 1;
int  ResInPlane  = 1;
int InPlaneOffset;
int BetPlanesOffset;

char BSFile[256], CorFile[256];
unsigned char cor[256][256][256];
short *bs;

int  OutStemSpec     = 0;
int  SubjectSpec     = 0;
int  SlicePlaneSpec  = 0;

long Nv,v;
FILE *fpBS,*fpCor;

int FirstSlice =  0;
int NSlices;
long nBSRows;
long nBSCols;
long nBSVox;
long nBSSlices;
long nCorVox;
size_t nRead,nWrite,nToRead,nToWrite;
long vbs,s,r,c,cs,cr,cc;

char *Progname ;

int main(int argc, char **argv)
{
  int n, ac,nargs;
  char **av, *cp;
  
  printf("&cor = %lx\n",cor);

  Progname = argv[0] ;
  if (argc == 1)    print_help() ;

  strcpy(Volume,"T1");

  ac = argc ;
  av = argv ;
  for ( ; argc > 1 && ISOPTION(*argv[1]) ; argc--, argv++)
  {
    nargs = get_option(argc, argv) ;
    argc -= nargs ;
    argv += nargs ;
  }
  if (argc != 1)    print_help() ;
  if (!OutStemSpec){
    fprintf(stderr,"ERROR: Must specify output stem\n");
    print_help() ;
  }
  if(!SubjectSpec){
    fprintf(stderr,"ERROR: Must specify a subject\n");
    print_help() ;
  }
  if(!SlicePlaneSpec){
    fprintf(stderr,"ERROR: Must specify a slice plane\n");
    print_help() ;
  }
  cp = getenv("SUBJECTS_DIR") ;
  if (!cp){
    fprintf(stderr,"%s: SUBJECTS_DIR not specified in the environment", 
	    Progname) ;
    exit(1);
  }
  strcpy(SubjectsDir, cp) ;

  nBSSlices = (long)(rint(256.0/ResBetPlanes));
  nBSRows   = (long)(rint(256.0/ResInPlane));  
  nBSCols   = nBSRows;
  nBSVox    = nBSRows*nBSCols;
  nToWrite = (size_t)(nBSVox/2);
  InPlaneOffset = (int)(rint(ResInPlane/2.0));
  BetPlanesOffset = (int)(rint(ResBetPlanes/2.0));

  printf("SubjectsDir   = %s\n",SubjectsDir);
  printf("Subject       = %s\n",Subject);  
  printf("Volume        = %s\n",Volume);
  printf("OutStem       = %s\n",OutStem);
  printf("Slice Plane   = %s (%d)\n",SlicePlane,SlicePlaneId);
  printf("Bet Plane Res = %d\n",ResBetPlanes);
  printf("Bet Plane Off = %d\n",BetPlanesOffset);
  printf("In  Plane Res = %d\n",ResInPlane);  
  printf("In  Plane Off = %d\n",InPlaneOffset);
  printf("nBSSlices     = %ld\n",nBSSlices);
  printf("nBSRows/Cols  = %ld\n",nBSRows);

  sprintf(CorDir,"%s/%s/mri/%s",SubjectsDir,Subject,Volume);
  printf("CorDir = %s\n",CorDir);

  nCorVox = 256L*256L;
  nToRead = (size_t)(nCorVox/2);
  
  printf("nCorVox = %ld\n",nCorVox);
  
  bs = (short *)malloc(sizeof(short)*nBSVox);
  if(bs == NULL){
    fprintf(stderr,"ERROR: could not allocate bs\n");
    exit(1);
  }
  
  /*************** Load COR files **************************/
  printf("&cor = %lx\n",cor);
  printf("Loading COR files \n");
  for(n=0;n<256;n++){
    sprintf(CorFile,"%s/COR-%03d",CorDir,n+1);
    printf("%03d ",n);
    if(n%16==1) printf("\n");
    fpCor = fopen(CorFile,"rb");
    if(fpCor == NULL){
      fprintf(stderr,"Can't open file %s\n", CorFile);
      fprintf(stderr,"%s\n",strerror(errno));
      exit(0);
    }
    nRead = fread(&cor[n][0][0], (size_t)1, nToRead, fpCor);
    if(nRead != nToRead){
      fprintf(stderr,"Read Failed on %s (%ld/%ld)\n",
	      CorFile,nRead,nToRead);
      exit(1);
    }
    nRead = fread(&cor[n][128][0], (size_t)1, nToRead, fpCor);
    if(nRead != nToRead){
      fprintf(stderr,"Read Failed on %s (%ld/%ld)\n",
	      CorFile,nRead,nToRead);
      exit(1);
    }
    fclose(fpCor);
  }
  printf("\n");  

  /********* Create Slices ***********************/
  for(s = 0; s < nBSSlices; s++){

    sprintf(BSFile,"%s_%03d.bshort",OutStem,s);
    printf("Creating %s\n",BSFile);
    fpBS = fopen(BSFile,"wb");
    if(fpBS == NULL){
      fprintf(stderr,"Can't open file %s\n", BSFile);
      fprintf(stderr,"%s\n",strerror(errno));
      exit(0);
    }
    
    memset(bs,NULL,nBSVox);

    cs = s*ResBetPlanes + BetPlanesOffset;  /* corronal slice number */

    for(r = 0; r < nBSRows; r++){    
      cr = r*ResInPlane + InPlaneOffset;       /* corronal row number */
      for(c = 0; c < nBSRows; c++){    

	cc = c*ResInPlane + InPlaneOffset;     /* corronal col number */
	vbs  = nBSCols*r + c;  
	bs[vbs] = (short)cor[cs][cr][cc];

        /*if(bs[vbs] != (short)0)
	printf("cor: %3d %3d %3d (%d)  bs: %2d %3d %3d (%d)\n",
	       cs,cr,cc, (int)cor[cs][cr][cc], 
	       s,  r, c, (int)bs[vbs]);*/
      }
    }
    
    nWrite = fwrite((void *)bs, (size_t)2, (size_t)nBSVox, fpBS);
    fclose(fpBS);
    if(nWrite != nBSVox){
      fprintf(stderr,"Write Failed on %s (%ld/%ld)\n",
	      BSFile,nWrite,nBSVox);
      exit(1);
    }

    sprintf(BSFile,"%s_%03d.hdr",OutStem,s);
    fpBS = fopen(BSFile,"w");
    if(fpBS == NULL){
      fprintf(stderr,"Can't open file %s\n", BSFile);
      fprintf(stderr,"%s\n",strerror(errno));
      exit(0);
    }
    fprintf(fpBS,"%d %d 1 0",nBSRows,nBSCols);
    fclose(fpBS);    
  }

  

  exit(0);
}
/**********************************************************/

static int get_option(int argc, char *argv[])
{
  int  nargs = 0 ;
  char *option ;

  if(argc < 3) print_help();
  
  option = argv[1] + 1 ;            /* past '-' */
  if (!strcmp(option, "-help"))
    print_help() ;
  else if (!strcmp(option, "-version"))
    print_version() ;
  else switch (tolower(*option))
  {
  case 'o':
    strcpy(OutStem,argv[2]) ;
    nargs = 1 ;
    OutStemSpec = 1;
    break ;
  case 's':
    strcpy(Subject,argv[2]) ;
    nargs = 1 ;
    SubjectSpec = 1;
    break ;
  case 'v':
    strcpy(Volume,argv[2]) ;
    nargs = 1 ;
    break ;
  case 'p':
    strcpy(SlicePlane,argv[2]) ;
    SlicePlaneId = check_slice_plane(SlicePlane);
    nargs = 1 ;
    SlicePlaneSpec = 1;
    nargs = 1 ;
    break ;
  case 'b':
    sscanf(argv[2],"%d",&ResBetPlanes);
    nargs = 1 ;
    break ;
  case 'i':
    sscanf(argv[2],"%d",&ResInPlane);
    nargs = 1 ;
    break ;
  default:
    fprintf(stderr, "unknown option %s\n", argv[1]) ;
    exit(1) ;
    break ;
  }

  return(nargs) ;
}

static void
usage_exit(void)
{
  print_usage() ;
  exit(1) ;
}

static void
print_usage(void)
{
  fprintf(stderr, 
          "usage: %s [options] -s subject -o outstem \n",Progname) ;
  fprintf(stderr, "\toptions are:\n") ;
  fprintf(stderr, "\t-v cor volume id <T1>\n") ;
  fprintf(stderr, "\t-p slice plane orientation (<ax>, cor, sag) \n") ;
  fprintf(stderr, "\t-b between plane resolution (mm) <1>\n") ;
  fprintf(stderr, "\t-i within plane resolution  (mm) <1>\n") ;
}

static void
print_help(void)
{
  print_usage() ;
  fprintf(stderr, 
          "\nThis program converts COR files to bshort, sub-sampling and re-orienting along the way.\n\n") ;
  exit(1) ;
}

static void print_version(void)
{
  fprintf(stderr, "$Id$ \n") ;
  exit(1) ;
}

int check_slice_plane(char *SlicePlane)
{
  
  if(!strcmp(SlicePlane,"ax"))  return(0);
  if(!strcmp(SlicePlane,"cor")) return(1);  
  if(!strcmp(SlicePlane,"sag")) return(2);

  fprintf(stderr,"Slice Plane %s unrecognized, use: ax, cor, or sag\n",
	  SlicePlane);
  exit(1);
  
}
