/* $Id */
#ifndef PKFUTIL_H
#define PKFUTIL_H

#ifdef	__cplusplus
extern "C" {
#endif
  /* ----------------------------------------------------- */

#include <stdio.h>

  /* volume subscript */
  typedef struct {
    int r, c, s;
  } VOLSS;

  /* voxel cluster */
  typedef struct {
    int nvxs;
    VOLSS *ss;
    int ipeak;
  } CLUSTER;

  /* cluster list */
  typedef struct {
    float valthresh;
    float distthresh;
    int   sizethresh;
    int   sorted;
    int   pksign;
    int   nbsize; /* neighborhood size */
    int   nclusters;
    CLUSTER **cluster;
  } CLUSTER_LIST, CLIST;

  /* image volume */
  typedef struct {
    int nrows, ncols, nslices;
    int nvox;
    float ***vox;
    float *data;
    char  ***voxflag; /* not used yet */
    char  *dataflag; /* not used yet */
    float rowres, colres, sliceres;
    float row_y0, col_x0, slice_z0;
  } IMGVOL;

  /* structure for sorting */
  typedef struct {
    int   id;
    float val;
  } SRTSTRUCT;

  int SuperThreshold(float v, float thresh, int sign);

  void *AllocCluster(int ncluster);
  int   FreeCluster(void **ppc);
  void *CopyCluster(void *dest, void *src);
  int   DumpCluster(FILE *fp, CLUSTER *c, IMGVOL *iv, char const *fmt);
  int   AddToCluster(VOLSS *ss, CLUSTER *cl);
  int IsClusterMember(VOLSS *ss, CLUSTER *cl);
  int GrowCluster(VOLSS *seed, CLUSTER *cl, IMGVOL *iv, 
		  float thresh, int sign, int ncallmax);
  int FindClusterPeak(CLUSTER *cl, IMGVOL *iv, int sign);

  void *AllocClusterList(int nclusters);
  void *CopyClusterListHeader(void *d, void *s);
  int   FreeClusterList(void **ppclist);
  int   DumpClusterList(FILE *fp, void *pclist, IMGVOL *iv, char const *fmt);
  int   AddToClusterList(CLUSTER *cl, CLUSTER_LIST *clist);
  int GrowClusterList(CLIST *clist, IMGVOL *iv, int nmax);

  CLUSTER_LIST *GetSuperVoxels(IMGVOL *iv, float valthresh, int pksign, int nbsize);
  CLUSTER_LIST *SortClusterList(CLIST *clist, IMGVOL *iv);
  int CompareSrtStruct(const void *s1, const void *s2); /* for qsort */
  CLUSTER_LIST *PruneByDist(CLIST *clist, float distthresh, IMGVOL *iv);
  CLUSTER_LIST *PruneBySize(CLIST *clist, int sizethresh);

  void   *AllocImgVol(int nslices, int nrows, int ncols);
  int     FreeImgVol(void **ppImgVol);
  int     DumpImgVol(FILE *fp, char const *fmt, void *ImgVol);
  IMGVOL *RandImgVol(int nslices, int nrows, int ncols,
		     float mean, float std, int seed);
  void   *CopyImgVol(void *dest, void *src);
  IMGVOL *ClusterList2ImgVol(CLIST *clist, IMGVOL *ivsrc, int roidef);

  IMGVOL *LoadBVolumeAsImgVol(char *stemtemplate, char *ext, int frame);
  int byteswapbuf(void *buf, long int nbufbytes);
  int PrintClusterReport(FILE *fp, CLIST *clist, IMGVOL *iv, 
		       char *infile, int no1988, char *CoorDef);
  int SaveImgVolAsBStack(char *stem, IMGVOL *iv);

  int ComputeTal(float *tal_x, float *tal_y, float *tal_z, 
		 int c, int r, int s, 
		 float col_x0, float row_y0, float slice_z0,
		 float colres, float rowres, float sliceres, 
		 int no1988, char *CoorDef);
       

#ifdef	__cplusplus
}
#endif /* #ifdef __cplusplus */

#endif /* #ifndef PKFUTIL_H */

