/***************************************************
 Name: synthw
 Purpose: synthesizes w file 
   (w files are the output of paint).
   ********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

int   fread1(int *v, FILE *fp) ;
int   fread2(int *v, FILE *fp) ;
int   fread3(int *v, FILE *fp) ;
int   fread4(float *v, FILE *fp) ;
float freadFloat(FILE *fp) ;
int fwrite1(int v,FILE *fp);
int fwrite2(int v,FILE *fp);
int fwrite3(int v,FILE *fp);
int fwrite4(int v,FILE *fp);
static double Gaussian01PDF(void);

char  wname[256];
float x,avg, std;
int   num;
int   i,ilat;
int   seed, nzero;
  
float f,r;
int   *vnum;
float *vval;
FILE *fp;
int   *exllist, nexl=0;
int   ismemb, j;

int main(int argc, char **argv)
{
  
  if(argc != 7){
    fprintf(stderr,"USAGE: synthw wfile nvertices avg std seed nzero\n");
    exit(1);
  }
  
  strcpy(wname,argv[1]);
  sscanf(argv[2],"%d",&num);
  sscanf(argv[3],"%f",&avg);
  sscanf(argv[4],"%f",&std);
  sscanf(argv[5],"%d",&seed);
  sscanf(argv[6],"%d",&nzero);
  
  srand48((long)seed);

  exllist = (int *)   malloc(nzero*sizeof(int));
  while(nexl < nzero){
    ismemb = 1;
    while(ismemb){
      j = (int)(rint(drand48()*num));
      ismemb = 0;
      for(i=0;i<nexl;i++) 
	if(j==exllist[i]) ismemb = 1;
      if(!ismemb){
	exllist[nexl] = j;
	nexl ++;
      }
    }
  }

  /*for(i=0;i<nexl;i++) 
    printf("%2d %4d\n",i,exllist[i]);*/

  /** open output file ****/
  fp = fopen(wname,"wb");
  if (fp==NULL) {
    fprintf(stderr,"File %s not found\n",wname);
    exit(1);
  }
  ilat = 0;
  fwrite2(ilat,fp);
  fwrite3(num-nzero, fp);

  for (i=0;i<num;i++)
  {
    ismemb = 0;
    for(j=0;j<nexl;j++) if(i==exllist[j]) ismemb = 1;
    if(ismemb) continue;

    fwrite3(i, fp);
    f = avg + std*Gaussian01PDF();
    fwrite(&f,4,1,fp);
  }
  fclose(fp);

  return(0);
  
}
/*************************************************************/


int
fread1(int *v, FILE *fp)
{
  unsigned char c;
  int  ret ;

  ret = fread(&c,1,1,fp);
  *v = c;
  return(ret) ;
}


int
fread2(int *v, FILE *fp)
{
  short s;
  int   ret ;

  ret = fread(&s,2,1,fp);
#ifdef Linux
  s = swapShort(s) ;
#endif
  *v = s;
  return(ret) ;
}

int
fread3(int *v, FILE *fp)
{
  unsigned int i = 0;
  int  ret ;

  ret = fread(&i,3,1,fp);
#ifdef Linux
  i = (unsigned int)swapInt(i) ;
#endif
  *v = ((i>>8) & 0xffffff);
  return(ret) ;
}

int
fread4(float *v, FILE *fp)
{
  float f;
  int   ret ;

  ret = fread(&f,4,1,fp);
#ifdef Linux
  f = swapFloat(f) ;
#endif
  *v = f;
  return(ret) ;
}


float
freadFloat(FILE *fp)
{
  float f;
  int   ret ;

  ret = fread(&f,4,1,fp);
#ifdef Linux
  f = swapFloat(f) ;
#endif
  return(f) ;
}

int
fwrite1(int v,FILE *fp)
{
  unsigned char c = (unsigned char)v;

  return(fwrite(&c,1,1,fp));
}

int
fwrite2(int v, FILE *fp)
{
  short s = (short)v;

#ifdef Linux
  s = swapShort(s) ;
#endif
  return(fwrite(&s,2,1,fp));
}

int
fwrite3(int v, FILE *fp)
{
  unsigned int i = (unsigned int)(v<<8);

#ifdef Linux
  i = (unsigned int)swapInt(i) ;
#endif
  return(fwrite(&i,3,1,fp));
}

int
fwrite4(int v,FILE *fp)
{
#ifdef Linux
  v = swapInt(v) ;
#endif
  return(fwrite(&v,4,1,fp));
}

/*********************************************************
 * Name:    Gaussian01PDF()
 * Purpose: generates random numbers that obey a gaussian 
 *          distribution with zero mean and std dev of 1:
 *              pdf(x) = e^(x^2/2)/sqrt(2pi)
 ************************************************************/
double Gaussian01PDF(void)
{
  double v1,v2,r2;

  do
    {
      v1 = 2.0 * drand48() - 1.0;
      v2 = 2.0 * drand48() - 1.0;
      r2 = v1*v1 + v2*v2;
    } 
  while( r2 > 1.0);

  return( v1 * sqrt( -2.0 * log(r2)/r2 ));
}

#if 0
MRISreadValues(MRI_SURFACE *mris, char *fname)
{
  int i,k,num,ilat;
  float f;
  float lat;
  FILE *fp;

  fp = fopen(fname,"r");
  if (fp==NULL) 
    ErrorReturn(ERROR_NOFILE, (ERROR_NOFILE,
                               "MRISreadValues: File %s not found\n",fname));
  fread2(&ilat,fp);
  lat = ilat/10.0;

  for (k=0;k<mris->nvertices;k++)
    mris->vertices[k].val=0;
  fread3(&num,fp);
  for (i=0;i<num;i++)
  {
    fread3(&k,fp);
    f = freadFloat(fp) ;
    if (k>=mris->nvertices||k<0)
      printf("MRISreadValues: vertex index out of range: %d f=%f\n",k,f);
    else
    {
      mris->vertices[k].val = f;
    }
  }
  fclose(fp);
  return(NO_ERROR) ;
}

#endif
