
/***************************************************************************
   talcoor.c (c) Randy Buckner 4/97  buckner@nmr.mgh.harvard.edu
***************************************************************************/ 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "talcoor.h"

#define TRUE 1
#define FALSE 0
#define STRLEN 200
#define SQR(x) ((x)*(x))
#define MAXIM 256
#define MAXSLICE 50
#define MAXPOINTS 350

int byteswapbuf(void *buf, long int nbufbytes);

int nperslice;
int xnum=0;
int ynum=0;
int allocated_avgim=FALSE;
int allocated_funim=FALSE;
int bufsize;

int point_num;

int t_x = 23;			/* THESE DEFINE THE TALAIRACH CENTER POINTS */ 
int t_y = 26;
int t_z = 24;

int draw_x, draw_y, draw_z;

int tal_x, tal_y, tal_z, diff_x, diff_y, diff_z;
float distance;

float diff_x2, diff_y2, diff_z2;
float distance2;

float pktx, pkty, pktz;
int   pki, pkj, pkk;

float x_res = 3.125;		/* The resolution of the images in each dimension */
float y_res = 3.125;
float z_res = 3.125;

float threshval;
float pval = .00001;		/* The p-value threshold for finding peaks */ 				 
float min_distance =  12.0;	/* The minimum distance between adjacent peaks */				 
int cluster_num = 8;	        /* The minimum number of significant voxels needed to define a peak */


int peak_x[MAXPOINTS], peak_y[MAXPOINTS], peak_z[MAXPOINTS], flag[MAXPOINTS], ncluster[MAXPOINTS] ;
int peak_ss[MAXPOINTS][3];
float peak_max[MAXPOINTS];

int nslices=39;
int firstslice=0;
int cluster = 0;

int replace = FALSE;

float weight;
 
int quiet = FALSE;
int helpflag = FALSE;
int clusterflag = FALSE;
int ksflag = FALSE;
int noheadflag = FALSE;
int selflag = FALSE, selavg = 8;
int simpleflag = FALSE;
int weightflag = TRUE;
int sortflag = TRUE;
int peakflag = TRUE;
int expandflag = FALSE;
int posflag = TRUE;
 
float **funim[MAXIM];
float **tmpim[MAXIM];
float **stkim[MAXIM];

int		i,j,k;

int 		peak, peak2, peak_count, peak_count2;
	
FILE *paraevnt;

char *parafile;

main(argc,argv)
int argc;
char *argv[];
{
    int i,j,k,slicenum,pn;
    char fname[STRLEN],*instem,*outstem;
    double num,sum,f;
    FILE *fp;	

    if (argc<2)
    {
      printf ("\ntalcoor version 1.0 - RL Buckner MGH-NMR (e-mail commments to: buckner@nmr.mgh.harvard.edu)\n");
      printf ("(C) Copyright 1997 Randy L. Buckner All Rights Reserved\n\n");
      printf("\n  Usage: %s [-options] -i <instem> \n",argv[0]);
      printf  ("Example: %s -nslices 16 -i S120r43\n\n", argv[0]);
      printf("    <instem>            : prefix of .ks.bfloat input files\n"); 
      printf("\nKey options:\n");
      printf("    -help               : get more detailed help\n");
      printf("    -firstslice <int>   : first slice number to process (default = 0)\n");
      printf("    -nslices <int>      : number of slices to process (default = 39)\n");
      printf("    -pos		      : create coordinate list based on positive changes (default)\n");
      printf("    -neg                : create coordinate list based on negative changes\n");
      printf("    -cluster <int>      : number of cluster voxels required\n");
      printf("    -pval <float>       : p-value threshold for finding peaks (default = .00001)\n");
      printf("    -distance <float>   : mm distance 2 points must be separated - else weight location by -logp (default = 12.0)\n");
      printf("    -noweight           : do not weight locations, simply keep the most significant\n");
      printf("    -nosort             : do not sort output by -logp value, default is to sort\n");
      printf("    -nopeaks            : do not produce ...peaks.bfloat file, default is to produce\n");
      printf("    -quiet              : quite mode\n");
      printf("    -nohead             : don't output header (good for bringing into stats packages)\n");
      printf("\nFormat options:\n    -kslogp             : use ...kslogp.bfloat not ...ks.bfloat\n");
      printf("    -selavg <int>       : use ...#.bfloat (where # is int) not ...ks.bfloat\n");
      printf("    -simple             : use dont use any fancy extension not ...ks.bfloat\n");
      printf("    -expand             : creates a separate peak bfloat for each slice, default is on concatenated file\n");
      printf("\nDimension options:\n");
      printf("    -xcenter <int>      : Talairach x-center (default = 23)\n");
      printf("    -ycenter <int>      : Talairach y-center (default = 26) NOTE: 12 mm are subtracted in output lists for 1988 atlas\n");
      printf("    -zcenter <int>      : Talairach z-center (default = 24)\n");
      printf("    -xres <float>       : resolution of input image in the x-dimension (default = 3.125)\n");
      printf("    -yres <float>       : resolution of input image in the y-dimension (default = 3.125)\n");
      printf("    -zres <float>       : resolution of input image in the z-dimension (default = 3.125)\n");
      exit(0);
    }

    for (i=1;i<argc;i++)
    {
      if (!strcmp("-help",argv[i])) helpflag = TRUE; 
      else if (!strcmp("-firstslice",argv[i])) firstslice = atoi(argv[++i]);
      else if (!strcmp("-nslices",argv[i])) nslices = atoi(argv[++i]);
      else if (!strcmp("-cluster",argv[i])) 
		{
		cluster_num = atoi(argv[++i]);
		clusterflag = TRUE;
		}
      else if (!strcmp("-selavg",argv[i])) 
		{
		selavg = atoi(argv[++i]);
		selflag = TRUE;
		}
      else if (!strcmp("-simple",argv[i])) simpleflag = TRUE;
      else if (!strcmp("-pval",argv[i])) pval = atof(argv[++i]);
      else if (!strcmp("-distance",argv[i])) min_distance = atof(argv[++i]);
      else if (!strcmp("-h",argv[i])) helpflag = TRUE;
      else if (!strcmp("-help",argv[i])) helpflag = TRUE;
      else if (!strcmp("-?",argv[i])) helpflag = TRUE;
      else if (!strcmp("-noweight",argv[i])) weightflag = FALSE;
      else if (!strcmp("-nopeaks",argv[i])) peakflag = FALSE;
      else if (!strcmp("-pos",argv[i])) posflag = TRUE;
      else if (!strcmp("-neg",argv[i])) posflag = FALSE;
      else if (!strcmp("-nosort",argv[i])) sortflag = FALSE;
      else if (!strcmp("-i",argv[i])) instem = argv[++i];
      else if (!strcmp("-o",argv[i])) outstem = argv[++i]; 
      else if (!strcmp("-loud",argv[i])) quiet = FALSE;
      else if (!strcmp("-expand",argv[i])) expandflag = TRUE;
      else if (!strcmp("-quiet",argv[i])) quiet = TRUE;
      else if (!strcmp("-nohead",argv[i])) noheadflag = TRUE;
      else if (!strcmp("-kslogp",argv[i])) ksflag = TRUE;
      else if (!strcmp("-xcenter",argv[i])) t_x = atoi(argv[++i]);
      else if (!strcmp("-ycenter",argv[i])) t_y = atoi(argv[++i]);
      else if (!strcmp("-zcenter",argv[i])) t_z = atoi(argv[++i]);
      else if (!strcmp("-xres",argv[i])) x_res = atof(argv[++i]);
      else if (!strcmp("-yres",argv[i])) y_res = atof(argv[++i]);
      else if (!strcmp("-zres",argv[i])) z_res = atof(argv[++i]);
      else
      {
        printf("unknown option %s\n",argv[i]);
        exit(0);
      }
    }

    if (helpflag == TRUE) 
		{
		display_help();
		exit(0);
		}

    threshval = -log(pval);
    cluster_num--; /* decrement the number because the peak itself will be part of the cluster */

   /* if (quiet == FALSE) 
	{
	printf ("\ntalcor version 1.0 - RL Buckner MGH-NMR\n");
        printf ("(C) Copyright 1997 Randy L. Buckner All Rights Reserved\n\n");
        } */

   if (quiet == FALSE && noheadflag == FALSE && posflag == TRUE) 
     printf ("INPUT FILE: %s  (signal change INCREASES)\n\n", instem);

   if (quiet == FALSE && noheadflag == FALSE && posflag == FALSE) 
     printf ("INPUT FILE: %s  (signal change DECREASES)\n\n", instem);

   if (posflag == FALSE) threshval = (threshval * -1);
 
/**********************/
/* Loads images file  */
/**********************/
   if (quiet == FALSE && noheadflag == FALSE) {
     printf("Searching for peaks more significant than p<%0.8f (%2.2f).\n", pval, (-log(pval)));
   }
        if (quiet == FALSE && clusterflag == TRUE && noheadflag == FALSE) printf("Peaks must be within a cluster of %d voxels.\n", (cluster_num+1));
	if (quiet == FALSE && noheadflag == FALSE && weightflag == FALSE) printf("Peaks must be separated by %2.1f mm or else only the largest is kept.\n", min_distance);
	if (quiet == FALSE && noheadflag == FALSE && weightflag == TRUE) printf("Peaks must be separated by %2.1f mm or else the weighted location is kept.\n", min_distance);
        if (quiet == FALSE && noheadflag == FALSE) printf("\n\t X\t Y\t Z\t-log(p)\n\t---\t---\t---\t-------\n");
       		peak_count = 0;
		/* OPEN FIRST .hdr FILE TO GET DIMENSIONS */

		if (simpleflag == TRUE) 
			{	
			sprintf(fname, "%s_000.hdr",instem);
			}
		else if (ksflag == FALSE ) 
			{
			sprintf(fname,"%s00.ks.hdr",instem); 	 /* makes name for first .hdr file  - */
			}
                else if (ksflag == TRUE ) 
			{
			sprintf(fname,"%s00.kslogp.hdr",instem); /* checks various flags to determine */
			}
                else if (selflag == TRUE ) 
			{
			sprintf(fname,"%s_000.%d.hdr",instem,selavg); /* the naming convention.  */
			}
  		fp = fopen(fname,"r");
  		if (fp==NULL)   {
			if (quiet == FALSE) printf("File %s not found.\n",fname);		/* .hdr doesn't exist ?? */
			exit(0);
				}
 	        fscanf(fp,"%d %d %d %*d\n",&ynum,&xnum,&nperslice); 				/* gets image dimensions!!! */
		if (nperslice != 1)   {
			if (quiet == FALSE) printf("Hmm. File %s contains more than one image. That's not right.\n",fname);
			exit(0);
				}
		fclose(fp);

		/* READ IN THE IMAGE VOLUME Z, Y, X */
		allocate_funim();
		for (k=0;k<nslices;k++) read_funim(instem,k);

		/* SEARCH THE VOLUME */
		/*printf("nslices = %2d, ynum = %2d, xnum = %2d\n",nslices,ynum,xnum);*/
     	 	for (k=0;k<nslices;k++){
		  for (i=0;i<ynum;i++)   {
		    for (j=0;j<xnum;j++)   {
		      
		      /* set the status of this point to not-a-peak */
		      peak = FALSE;
		      
                      /* get value for this point; this is is the actual -logp value at Z,X,Y */
		      f = funim[k][i][j]; 
		      /* look for increases */
		      if (posflag == TRUE){ 
			
                        /* Its above the threshold so it might be a peak... */
			if (f > threshval) peak = TRUE;                      

			/* Ah, but is it the biggest of its neighbors? */ 
			if (k > 0) if (funim[k-1][i][j] > f) peak = FALSE;         
			if (k < nslices-1) if (funim[k+1][i][j] > f) peak = FALSE;
			if (i > 0) if (funim[k][i-1][j] > f) peak = FALSE;
			if (i < ynum-1) if (funim[k][i+1][j] > f) peak = FALSE;
			if (j > 0) if (funim[k][i][j-1] > f) peak = FALSE;
			if (j < xnum-1) if (funim[k][i][j+1] > f) peak = FALSE;
			
			if (k > 0 && i > 0) if (funim[k-1][i-1][j] > f) peak = FALSE;
			if (k < nslices-1 && i > 0) if (funim[k+1][i-1][j] > f) peak = FALSE;
			if (k > 0 && i < ynum-1) if (funim[k-1][i+1][j] > f) peak = FALSE;
			if (k < nslices-1 && i < ynum-1) if (funim[k+1][i+1][j] > f) peak = FALSE;
			if (k < nslices-1 && j < xnum-1) if (funim[k+1][i][j+1] > f) peak = FALSE;
			if (k > 0 && j < xnum-1) if (funim[k-1][i][j+1] > f) peak = FALSE;
			if (k > 0 && j > 0) if (funim[k-1][i][j-1] > f) peak = FALSE;
			if (k < nslices-1 && j > 0) if (funim[k+1][i][j-1] > f) peak = FALSE;
			if (i < ynum-1 && j < xnum-1) if (funim[k][i+1][j+1] > f) peak = FALSE;
			if (i > 0 && j < xnum-1) if (funim[k][i-1][j+1] > f) peak = FALSE;
			if (i > 0 && j > 0) if (funim[k][i-1][j-1] > f) peak = FALSE;
			if (i < ynum-1 && j > 0) if (funim[k][i+1][j-1] > f) peak = FALSE;
			
			if (k < nslices-1 && i > 0 && j > 0) if (funim[k+1][i-1][j-1] > f) peak = FALSE;
			if (k > 0 && i < ynum-1 && j > 0) if (funim[k-1][i+1][j-1] > f) peak = FALSE;
			if (k < nslices-1 && i < ynum-1 && j > 0) if (funim[k+1][i+1][j-1] > f) peak = FALSE;
			if (k < nslices-1 && i > 0 && j < xnum-1) if (funim[k+1][i-1][j+1] > f) peak = FALSE;
			if (k > 0 && i < ynum-1 && j < xnum-1) if (funim[k-1][i+1][j+1] > f) peak = FALSE;
			if (k < nslices-1 && i < ynum-1 && j < xnum-1) if (funim[k+1][i+1][j+1] > f) peak = FALSE;
			if (k > 0 && i > 0 && j > 0) if (funim[k-1][i-1][j-1] > f) peak = FALSE;
			if (k > 0 && i > 0 && j < xnum-1) if (funim[k-1][i-1][j+1] > f) peak = FALSE; 

			/* Check THE CLUSTER CONSTRAINT */
			if (clusterflag == TRUE){		   
			  /* assume cluster size is zero */
			  cluster = 0;

			  /* look at the nearest adjacent neighbors */
			  if (k > 0) if (funim[k-1][i][j] > threshval) cluster++;                   
			  if (k < nslices-1) if (funim[k+1][i][j] > threshval) cluster++;
			  if (i > 0) if (funim[k][i-1][j] > threshval) cluster++;
			  if (i < ynum-1) if (funim[k][i+1][j] > threshval) cluster++;
			  if (j > 0) if (funim[k][i][j-1] > threshval) cluster++;
			  if (j < xnum-1) if (funim[k][i][j+1] > threshval) cluster++;
			  
			  if (k > 0 && i > 0) if (funim[k-1][i-1][j] > threshval) cluster++;
			  if (k < nslices-1 && i > 0) if (funim[k+1][i-1][j] > threshval) cluster++;
			  if (k > 0 && i < ynum-1) if (funim[k-1][i+1][j] > threshval) cluster++;
			  if (k < nslices-1 && i < ynum-1) if (funim[k+1][i+1][j] > threshval) cluster++;
			  if (k < nslices-1 && j < xnum-1) if (funim[k+1][i][j+1] > threshval) cluster++;
			  if (k > 0 && j < xnum-1) if (funim[k-1][i][j+1] > threshval) cluster++;
			  if (k > 0 && j > 0) if (funim[k-1][i][j-1] > threshval) cluster++;
			  if (k < nslices-1 && j > 0) if (funim[k+1][i][j-1] > threshval) cluster++;
			  if (i < ynum-1 && j < xnum-1) if (funim[k][i+1][j+1] > threshval) cluster++;
			  if (i > 0 && j < xnum-1) if (funim[k][i-1][j+1] > threshval) cluster++;
			  if (i > 0 && j > 0) if (funim[k][i-1][j-1] > threshval) cluster++;
			  if (i < ynum-1 && j > 0) if (funim[k][i+1][j-1] > threshval) cluster++;
			  
			  if (k < nslices-1 && i > 0 && j > 0) if (funim[k+1][i-1][j-1] > threshval) cluster++;
			  if (k > 0 && i < ynum-1 && j > 0) if (funim[k-1][i+1][j-1] > threshval) cluster++;
			  if (k < nslices-1 && i < ynum-1 && j > 0) if (funim[k+1][i+1][j-1] > threshval) cluster++;
			  if (k < nslices-1 && i > 0 && j < xnum-1) if (funim[k+1][i-1][j+1] > threshval) cluster++;
			  if (k > 0 && i < ynum-1 && j < xnum-1) if (funim[k-1][i+1][j+1] > threshval) cluster++;
			  if (k < nslices-1 && i < ynum-1 && j < xnum-1) if (funim[k+1][i+1][j+1] > threshval) cluster++;
			  if (k > 0 && i > 0 && j > 0) if (funim[k-1][i-1][j-1] > threshval) cluster++;
			  if (k > 0 && i > 0 && j < xnum-1) if (funim[k-1][i-1][j+1] > threshval) cluster++;

			  if (cluster < cluster_num ) peak = FALSE;
			}/* endif clusterflag == TRUE */
		      } /* endif posflag == TRUE */

		      /* look for decreases */
		      if (posflag == FALSE){         	   
			/* Its below the negative value threshold so it might be a peak... */
			if (f < threshval) peak = TRUE;                   
			
			/* Ah, but is it the smallest of its neighbors? */ 
			if (k > 0) if (funim[k-1][i][j] < f) peak = FALSE;         
			if (k < nslices-1) if (funim[k+1][i][j] < f) peak = FALSE;
			if (i > 0) if (funim[k][i-1][j] < f) peak = FALSE;
			if (i < ynum-1) if (funim[k][i+1][j] < f) peak = FALSE;
			if (j > 0) if (funim[k][i][j-1] < f) peak = FALSE;
			if (j < xnum-1) if (funim[k][i][j+1] < f) peak = FALSE;
			
			if (k > 0 && i > 0) if (funim[k-1][i-1][j] < f) peak = FALSE;
			if (k < nslices-1 && i > 0) if (funim[k+1][i-1][j] < f) peak = FALSE;
			if (k > 0 && i < ynum-1) if (funim[k-1][i+1][j] < f) peak = FALSE;
			if (k < nslices-1 && i < ynum-1) if (funim[k+1][i+1][j] < f) peak = FALSE;
			if (k < nslices-1 && j < xnum-1) if (funim[k+1][i][j+1] < f) peak = FALSE;
			if (k > 0 && j < xnum-1) if (funim[k-1][i][j+1] < f) peak = FALSE;
			if (k > 0 && j > 0) if (funim[k-1][i][j-1] < f) peak = FALSE;
			if (k < nslices-1 && j > 0) if (funim[k+1][i][j-1] < f) peak = FALSE;
			if (i < ynum-1 && j < xnum-1) if (funim[k][i+1][j+1] < f) peak = FALSE;
			if (i > 0 && j < xnum-1) if (funim[k][i-1][j+1] < f) peak = FALSE;
			if (i > 0 && j > 0) if (funim[k][i-1][j-1] < f) peak = FALSE;
			if (i < ynum-1 && j > 0) if (funim[k][i+1][j-1] < f) peak = FALSE;
			
			if (k < nslices-1 && i > 0 && j > 0) if (funim[k+1][i-1][j-1] < f) peak = FALSE;
			if (k > 0 && i < ynum-1 && j > 0) if (funim[k-1][i+1][j-1] < f) peak = FALSE;
			if (k < nslices-1 && i < ynum-1 && j > 0) if (funim[k+1][i+1][j-1] < f) peak = FALSE;
			if (k < nslices-1 && i > 0 && j < xnum-1) if (funim[k+1][i-1][j+1] < f) peak = FALSE;
			if (k > 0 && i < ynum-1 && j < xnum-1) if (funim[k-1][i+1][j+1] < f) peak = FALSE;
			if (k < nslices-1 && i < ynum-1 && j < xnum-1) if (funim[k+1][i+1][j+1] < f) peak = FALSE;
			if (k > 0 && i > 0 && j > 0) if (funim[k-1][i-1][j-1] < f) peak = FALSE;
			if (k > 0 && i > 0 && j < xnum-1) if (funim[k-1][i-1][j+1] < f) peak = FALSE;
			
			/* ADD THE CLUSTER CONSTRAINT */
			if (clusterflag == TRUE)  {
			  /* assume cluster size of zero */
			  cluster = 0;
			  
			  /* look at the nearest adjacent neighbors */
			  if (k > 0) if (funim[k-1][i][j] < threshval) cluster++;                   
			  if (k < nslices-1) if (funim[k+1][i][j] < threshval) cluster++;
			  if (i > 0) if (funim[k][i-1][j] > threshval) cluster++;
			  if (i < ynum-1) if (funim[k][i+1][j] < threshval) cluster++;
			  if (j > 0) if (funim[k][i][j-1] < threshval) cluster++;
			  if (j < xnum-1) if (funim[k][i][j+1] < threshval) cluster++;

			  if (k > 0 && i > 0) if (funim[k-1][i-1][j] < threshval) cluster++;
			  if (k < nslices-1 && i > 0) if (funim[k+1][i-1][j] < threshval) cluster++;
			  if (k > 0 && i < ynum-1) if (funim[k-1][i+1][j] < threshval) cluster++;
			  if (k < nslices-1 && i < ynum-1) if (funim[k+1][i+1][j] < threshval) cluster++;
			  if (k < nslices-1 && j < xnum-1) if (funim[k+1][i][j+1] < threshval) cluster++;
			  if (k > 0 && j < xnum-1) if (funim[k-1][i][j+1] < threshval) cluster++;
			  if (k > 0 && j > 0) if (funim[k-1][i][j-1] < threshval) cluster++;
			  if (k < nslices-1 && j > 0) if (funim[k+1][i][j-1] < threshval) cluster++;
			  if (i < ynum-1 && j < xnum-1) if (funim[k][i+1][j+1] < threshval) cluster++;
			  if (i > 0 && j < xnum-1) if (funim[k][i-1][j+1] < threshval) cluster++;
			  if (i > 0 && j > 0) if (funim[k][i-1][j-1] < threshval) cluster++;
			  if (i < ynum-1 && j > 0) if (funim[k][i+1][j-1] < threshval) cluster++;
			  
			  if (k < nslices-1 && i > 0 && j > 0) if (funim[k+1][i-1][j-1] < threshval) cluster++;
			  if (k > 0 && i < ynum-1 && j > 0) if (funim[k-1][i+1][j-1] < threshval) cluster++;
			  if (k < nslices-1 && i < ynum-1 && j > 0) if (funim[k+1][i+1][j-1] < threshval) cluster++;
			  if (k < nslices-1 && i > 0 && j < xnum-1) if (funim[k+1][i-1][j+1] < threshval) cluster++;
			  if (k > 0 && i < ynum-1 && j < xnum-1) if (funim[k-1][i+1][j+1] < threshval) cluster++;
			  if (k < nslices-1 && i < ynum-1 && j < xnum-1) if (funim[k+1][i+1][j+1] < threshval) cluster++;
			  if (k > 0 && i > 0 && j > 0) if (funim[k-1][i-1][j-1] < threshval) cluster++;
			  if (k > 0 && i > 0 && j < xnum-1) if (funim[k-1][i-1][j+1] < threshval) cluster++;
			  
			  if (cluster < cluster_num ) peak = FALSE;
			} /* endif clusterflag == TRUE */
		      } /* endif looking for signal decreases */
		      
		      tal_x = (int)((j - t_x) * x_res); 			/* convert to t88 coor */
		      tal_y = (int)((t_y - i) * y_res) - 12;			/* correct by 12 for atlas */
		      tal_z = (int)((t_z - k) * z_res);
		      
		      pktx = ((j - t_x) * x_res); 			/* convert to t88 coor */
		      pkty = ((t_y - i) * y_res) - 12;			/* correct by 12 for atlas */
		      pktz = ((t_z - k) * z_res);
		      pkj  = j;
		      pki  = i;
		      pkk  = k;
		      
		      if (peak == TRUE){
			flag[peak_count+1] = 1;		
			/* Check if there's a point near it */ 	
			for (pn=1; pn < (peak_count+1); pn++){
			  diff_x = sqrt((peak_x[pn]-tal_x)*(peak_x[pn]-tal_x));
			  diff_y = sqrt((peak_y[pn]-tal_y)*(peak_y[pn]-tal_y));
			  diff_z = sqrt((peak_z[pn]-tal_z)*(peak_z[pn]-tal_z));
			  distance = sqrt((diff_x*diff_x) + (diff_y*diff_y) + (diff_z*diff_z)); 
  		   
			  diff_x2 = (peak_ss[pn][1]-j)*x_res;
			  diff_y2 = (peak_ss[pn][0]-i)*y_res;
			  diff_z2 = (peak_ss[pn][2]-k)*z_res;
			  distance2 = sqrt((diff_x2*diff_x2) + (diff_y2*diff_y2) + (diff_z2*diff_z2)); 

			  /*if (distance < min_distance){*/
			  if (distance2 < min_distance){
			    /* We have overlap. must eliminate one of the peaks */
			    flag[peak_count+1] = 2;
			    if (abs(f) > abs(peak_max[pn]) || f == peak_max[pn]){
			      /* The new point is more or EQUALLY significant, kill old */

			      if (weightflag == TRUE){
				/* make new location, weighted location */		
				weight = (peak_max[pn]/(f*2));
				tal_x = (int)(tal_x+(weight*(peak_x[pn]-tal_x)));
				tal_y = (int)(tal_y+(weight*(peak_y[pn]-tal_y)));
				tal_z = (int)(tal_z+(weight*(peak_z[pn]-tal_z)));
			      }
			      peak_x[pn] = 0;
			      peak_y[pn] = 0;
			      peak_z[pn] = 0;
			      peak_max[pn] = 0;
			      flag[pn] = 99;  /* e.g., eliminate this point */						
			    }
			    if (abs(peak_max[pn]) > abs(f)){
			      /* Old is more significant, don't add new */

			      if (weightflag == TRUE){
				/* make new location, weighted location */
				weight = (f/(peak_max[pn]*2));
				peak_x[pn] = (int)(peak_x[pn]+(weight*(tal_x-peak_x[pn])));
				peak_y[pn] = (int)(peak_y[pn]+(weight*(tal_y-peak_y[pn])));
				peak_z[pn] = (int)(peak_z[pn]+(weight*(tal_z-peak_z[pn])));
				flag[pn] = 3;
			      }
			      peak = FALSE;
			      break; 
			    }
			  } /* end if dist < distmin */
			} /* end for pn */
		      } /* endif peak==TRUE */			
		      
		      if (peak == TRUE){
			/* ITS A TRUE PEAK, LOG IT!! */
			peak_count++;
			peak_x[peak_count] = tal_x;
			peak_y[peak_count] = tal_y;
			peak_z[peak_count] = tal_z;
			peak_max[peak_count] = f; 
                        ncluster[peak_count] = cluster;
                        peak_ss[peak_count][0] = pki;
                        peak_ss[peak_count][1] = pkj;
                        peak_ss[peak_count][2] = pkk;
			/*printf("%3d   %6.4f %3d %3d %3d\n",peak_count,f,pkk, pkj, pki);*/
		      }
		    } /* end for over j (x) */
		  } /* end for over i (y) */
		} /* end for over k (z) */

		/* Should we sort it ? */
		if (sortflag == TRUE) 
		  sort(peak_count, peak_x, peak_y, peak_z, peak_max, flag, peak_ss);        

		point_num = 1;
		for (pn = peak_count; pn > 0; pn--){
		  /* ok, spit them out ... */
		  if (quiet == FALSE && abs(peak_max[pn]) > 0) 
		    {
		      printf ("pk%d)\t%3d\t%3d\t%3d\t %6.4f \t %3d %3d %3d \t %3d  \n", 
			      point_num, peak_x[pn], peak_y[pn], peak_z[pn], peak_max[pn],
			      peak_ss[pn][2],peak_ss[pn][0],peak_ss[pn][1],ncluster[pn]);
		      point_num++;
		    }
		  
		  if (peakflag == TRUE && abs(peak_max[pn]) > 0) 
		    {
		      draw_x = (int)((peak_x[pn]/x_res)+t_x);
		      draw_y = (int)(t_y-((peak_y[pn]+12)/y_res));
		      draw_z = (int)(t_z-(peak_z[pn]/z_res));
		      /*printf("%3d   %3d %3d %3d\n",pn,draw_z,draw_x,draw_y);*/
		      if (draw_x == 0) draw_x = 1; /* a fudge for plotting */
		      if (draw_y == 0) draw_y = 1;
		      if (draw_x == xnum) draw_x = xnum-1;
		      if (draw_y == ynum) draw_y = ynum-1;
		      
		      if (draw_x < 1 || draw_y < 1 || draw_z < 0 || draw_z > nslices || draw_x > (xnum-1) || draw_y > (ynum-1))
			{ 
			  if (quiet == FALSE) printf("Cannot plot %d, %d, %d (%d,%d,%d)\n", 
						     peak_x[pn], peak_y[pn], peak_z[pn], draw_x, draw_y, draw_z);
			}
		      else
			{								/* Adds new peak to peak output .bfloat */
			  tmpim[draw_z][draw_y][draw_x] = peak_max[pn];
			  tmpim[draw_z][draw_y-1][draw_x-1] = peak_max[pn];
			  tmpim[draw_z][draw_y+1][draw_x+1] = peak_max[pn];
			  tmpim[draw_z][draw_y-1][draw_x+1] = peak_max[pn];
			  tmpim[draw_z][draw_y+1][draw_x-1] = peak_max[pn];
			  tmpim[draw_z][draw_y][draw_x-1] = peak_max[pn];
			  tmpim[draw_z][draw_y][draw_x+1] = peak_max[pn];
			  tmpim[draw_z][draw_y-1][draw_x] = peak_max[pn];
			  tmpim[draw_z][draw_y+1][draw_x] = peak_max[pn];
			}
		    }
		}
		
		
		if (peakflag == TRUE && expandflag == TRUE) /* Creates peak bfloats, one file per slice */
		    for (k=0;k<nslices;k++) write_tmpim(instem,k);


		if (peakflag == TRUE && expandflag == FALSE) /* Creates peak bfloats, one big file with all the slices */
		  write_long_tmpim(instem,nslices);		
		   
		exit(0);
}
/*-----------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------------------*/
    
allocate_funim()
{
  int i,k;

  for (k=0;k<nslices;k++)
  {
    funim[k] = (float **)calloc(ynum,sizeof(float *));
    tmpim[k] = (float **)calloc(ynum,sizeof(float *));
    stkim[k] = (float **)calloc(ynum,sizeof(float *));
    for (i=0;i<ynum;i++)
    {
      funim[k][i] = (float *)calloc(xnum,sizeof(float));
      tmpim[k][i] = (float *)calloc(xnum,sizeof(float));
      stkim[k][i] = (float *)calloc(xnum,sizeof(float));
    }
  }
  allocated_funim = TRUE;
}

/* --------------------------------------------------- */
int read_funim(fstem,slicenum)
char *fstem;
int slicenum;
{
  int i,j,k;
  char fname[STRLEN];
  FILE *fp;
  
  if (simpleflag == TRUE) 
    sprintf(fname,"%s_0%02d",fstem,slicenum);
  else if (ksflag == FALSE) 
    sprintf(fname,"%s%02d.ks",fstem,slicenum);
  else if (ksflag == TRUE) 
    sprintf(fname,"%s%02d.kslogp",fstem,slicenum);
  else if (selflag == TRUE) 
    sprintf(fname,"%s_0%02d.%d",fstem,slicenum,selavg);

  read_bfloat(fname,funim,xnum,ynum,slicenum);

  return(0);
}

/* --------------------------------------------------- */
int write_tmpim(fstem,slicenum)
char *fstem;
int slicenum;
{
  int i,j,k;
  char fname[STRLEN];
  FILE *fp;
  
  sprintf(fname,"%s%02d.peaks",fstem,slicenum);
  write_bfloat(fname,tmpim,xnum,ynum,slicenum);
  return(0);
}

/* --------------------------------------------------- */
int write_long_tmpim(fstem,slices)
char *fstem;
int slices;
{
  int i,j,k;
  char fname[STRLEN];
  FILE *fp;
  
  sprintf(fname,"%s.peaks",fstem);
  write_long_bfloat(fname,tmpim,xnum,ynum,slices);
  return(0);
}

/* --------------------------------------------------- */
int print_error(str)
char *str;
{
  printf(str);
  exit(0);
  return(0);
}
/* --------------------------------------------------- */
char *lmalloc(size)
unsigned long size;
{
  char *p;
  
  p = malloc(size);
  if (p==NULL) print_error("Cannot malloc()\n");
  return p;
}

/* --------------------------------------------------- */
char *lcalloc(nmemb,size)
size_t nmemb,size;
{
  char *p;
  
  p = calloc(nmemb,size);
  if (p==NULL) print_error("Cannot calloc()\n");
  return p;
}

/* --------------------------------------------------- */
int file_name(fpref,fname,num,form)
char *fpref,*fname,*form;
int num;
{
  char ext[10];
  
  sprintf(ext,form,num);
  strcpy(fname,fpref);
  strcat(fname,ext);
  return(0);
}
  
/* --------------------------------------------------- */
int sort (tn, tx, ty, tz, tmax, tflag, tss)
     int tn;
     int tx[MAXPOINTS], ty[MAXPOINTS], tz[MAXPOINTS];
     int tss[MAXPOINTS][3];
     float tmax[MAXPOINTS];
     int tflag[MAXPOINTS];
{
  unsigned long i,j,inc;
  float vmax;
  int vx,vy,vz,vflag;
  int vss[3];
  inc = 1;
  do {
    inc *= 3;
    inc++;
  } while (inc <=tn);
  
  do {
    inc /=3;
    for (i=inc+1;i<=tn;i++) {
      vmax=tmax[i];
      vx = tx[i];
      vy = ty[i];
      vz = tz[i];
      vflag = tflag[i];
      vss[0] = tss[i][0];
      vss[1] = tss[i][1];
      vss[2] = tss[i][2];
      j=i;
      while (tmax[j-inc] > vmax) {
	tmax[j]=tmax[j-inc];
	tx[j]=tx[j-inc];
	ty[j]=ty[j-inc];
	tz[j]=tz[j-inc];
	tflag[j]=tflag[j-inc];
	tss[j][0] = tss[j-inc][0] ;
	tss[j][1] = tss[j-inc][1] ;
	tss[j][2] = tss[j-inc][2] ;
	j -= inc;
	if (j <= inc) break;
      }
      tmax[j] = vmax;
      tx[j] = vx;
      ty[j] = vy;
      tz[j] = vz;
      tflag[j] = vflag;
      tss[j][0] = vss[0];
      tss[j][1] = vss[1];
      tss[j][2] = vss[2];
    }
  } while (inc>1);
  
  for (i=1; i < (tn+1); i++){ 
      peak_max[i] = tmax[i];
      peak_y[i] = ty[i];
      peak_x[i] = tx[i];
      peak_z[i] = tz[i];
      flag[i] = tflag[i];
  }
  return(0);
}

/* --------------------------------------------------- */
int read_bfloat(fpref,im,xnum,ynum,k)
char *fpref;
float ***im;
int xnum,ynum,k;
{
  int i,j,bufsize, endian,swap=0;
  char fname[STRLEN];
  FILE *fp;
  int nread,txnum,tynum,tnperslice;
  static float *fbuf;
  static int fbufsize = -1;

  sprintf(fname,"%s.hdr",fpref);
  fp = fopen(fname,"r");
  if (fp==NULL) {printf("File %s not found\n",fname);exit(0);}
  fscanf(fp,"%d %d %d %d\n",&tynum,&txnum,&tnperslice,&endian);
  sprintf(fname,"%s.bfloat",fpref);
  fp = fopen(fname,"rb");
  if (fp==NULL) {printf("File %s not found\n",fname);exit(0);}
  bufsize = xnum*ynum;
  if (fbufsize<bufsize)
    {
      if (fbufsize>0) free(fbuf);
      fbufsize = bufsize;
      fbuf = (float *)calloc(bufsize,sizeof(float));
    }
  nread = fread(fbuf,sizeof(float),bufsize,fp);
#ifdef Linux
  if(endian==0) swap = 1;
#else
  if(endian==1) swap = 1;
#endif
  if(swap) {
    byteswapbuf(fbuf,bufsize*sizeof(float));
  }
  for (i=0;i<ynum;i++)
    for (j=0;j<xnum;j++)
      {
	im[k][i][j] = fbuf[i*xnum+j];
	if ((!finite(im[k][i][j]))||(!finite(fbuf[i*xnum+j])));
	
      }
  fclose(fp);
  return(0);
}
/* ---------------------------------------------- */
int byteswapbuf(void *buf, long int nbufbytes)
{
  register char *cbuf,c;
  register long int n, nmax;

  nmax = nbufbytes;
  cbuf = (char *)buf;
  for(n=0;n<nmax;n+=4){
    c = *cbuf;
    *cbuf = *(cbuf+3);
    *(cbuf+3) = c;

    c = *(cbuf+1);
    *(cbuf+1) = *(cbuf+2);
    *(cbuf+2) = c;

    cbuf += 4;
  }
  return(0);
}

write_bfloat(fpref,im,xnum,ynum,k)
char *fpref;
float ***im;
int xnum,ynum,k;
{
  int i,j,bufsize;
  char fname[STRLEN];
  FILE *fp;
  static float *fbuf;
  static int fbufsize = -1;

  sprintf(fname,"%s.hdr",fpref);
  fp = fopen(fname,"w");
  if (fp==NULL) {printf("Can't open file %s\n",fname);exit(0);}
  fprintf(fp,"%d %d %d 0\n",ynum,xnum,nperslice);
  fclose(fp);
  sprintf(fname,"%s.bfloat",fpref);
  fp = fopen(fname,"wb");
  if (fp==NULL) {printf("Can't open file %s\n",fname);exit(0);}
  bufsize = xnum*ynum;
  if (fbufsize<bufsize)
  {
    if (fbufsize>0) free(fbuf);
    fbufsize = bufsize;
    fbuf = (float *)calloc(bufsize,sizeof(float));
  }
    
    for (i=0;i<ynum;i++)
    for (j=0;j<xnum;j++)
    {
      fbuf[i*xnum+j] = im[k][i][j];
      if ((!finite(im[k][i][j]))||(!finite(fbuf[i*xnum+j])));
    }
    fwrite(fbuf,sizeof(float),bufsize,fp);
    fclose(fp);
}

write_long_bfloat(fpref,im,xnum,ynum,nperslice)
char *fpref;
float ***im;
int xnum,ynum,nperslice;
{
  int i,j,k,bufsize;
  char fname[STRLEN];
  FILE *fp;
  static float *fbuf;
  static int fbufsize = -1;

  sprintf(fname,"%s.hdr",fpref);
  fp = fopen(fname,"w");
  if (fp==NULL) {printf("Can't open file %s\n",fname);exit(0);}
  fprintf(fp,"%d %d %d 0\n",ynum,xnum,nperslice);
  fclose(fp);
  sprintf(fname,"%s.bfloat",fpref);
  fp = fopen(fname,"wb");
  if (fp==NULL) {printf("Can't open file %s\n",fname);exit(0);}
  bufsize = xnum*ynum;
  if (fbufsize<bufsize)
  {
    if (fbufsize>0) free(fbuf);
    fbufsize = bufsize;
    fbuf = (float *)calloc(bufsize,sizeof(float));
  }
  for (k=0;k<nperslice;k++)
  {
    for (i=0;i<ynum;i++)
    for (j=0;j<xnum;j++)
    {
      fbuf[i*xnum+j] = im[k][i][j];
      if ((!finite(im[k][i][j]))||(!finite(fbuf[i*xnum+j])))
        printf("write_bfloat(%s): illegal float (k=%d,i=%d,j=%d): %f,%f\n",
               fname,k,i,j,im[k][i][j],fbuf[i*xnum+j]);
    }
    fwrite(fbuf,sizeof(float),bufsize,fp);
  }
  fclose(fp);
}

display_help()
{
printf("\ntalcoor version 1.0 (C) 1997 Randy L. Buckner e-mail comments to buckner@nmr.mgh.harvard.edu\n\n");
printf("\nThe purpose of this program is to take in .bfloat images that contain statistical\n");
printf("probability values (-log[p]) and find the locations of the peaks in Talairach and Tournoux\n");
printf("1988 atlas space.\n\n");
printf("The program, essentially, screens for peaks based on the following criteria:\n\n");
printf("    (1) The voxel must reach a certain threshold specified by the -pval <float> option. The float\n");
printf("        should be the actual p-value (e.g., \"-pval .0001\").\n\n");
printf("    (2) The voxel must be the largest of its neighbors.\n\n");
printf("    (3) If the -cluster <int> constraint is set, its neighbors must contain at least <int>\n");
printf("        members that reach the critical p-value (e.g., \"-cluster 8\").\n\n");
printf("    (4) If the -distance <float> constraint is set then peaks within <float> mm are not\n");
printf("        considered. In these instances the weighted location is kept unless the -noweight option\n");
printf("        option is selected. Then, the largest peak is kept. (e.g., \"-distance 8.0 -noweight\")\n\n");
printf("The ouput of the program is twofold. First, a text list of all identified peaks are\n");
printf("generated in a file with the .coor extension. These peaks are numbered and sorted\n");
printf("largest to smallest unless the -nosort option is used. Then they appear in the order\n");
printf("they were detected. The second item produced is a .bfloat image stack that contains\n");
printf("one image for each slice of the brain. A square dot is placed in these images\n");
printf("corresponding to the identified peaks. The intenisty value of these dots is the p-value.\n");
printf("These stacks can be viewed by %%xds <name>.peaks.bfloat &. Alternatively, these stacks\n");
printf("can be placed in a mosaic and viewed on top of an anatomic image. Yet another alternative,\n");
printf("which can be used to decide whether the peak search algorith is working, is to display the\n");
printf("mosaic of the .peaks.bfloat file on top of a mosaic of the statistical map that produced\n");
printf("it (e.g., %%xds -A mosTAL_AVG10stemallfix.peaks.bfloat mosTAL_AVG10stemallfix.ks.bfloat &).\n\n");
printf("talcoor is meant (but not required) to be used with talreg and taltime. \"talreg\" will take\n");
printf("in coordinates identified by talcoor and automatically define regions around it. \"taltime\"\n");
printf("will take in a region and get its timecourse. Still yet another program \"normalizer\" will\n");
printf("take in the timeseries produced by taltime and drift correct, smooth, and convert it to\n");
printf("percent change. Collectively, these programs can thus be used to automatically interogate\n");
printf("fMRI timecourse data.\n\n");
printf("Options:\n\n");
printf("    -expand    This program, by default, makes a single .bfloat stack that contains all peaks.\n");
printf("               Other programs output one image per slice so that they can be made to correspond\n");
printf("               to single slice data such as .bshort timeseries. Choosing the \"-expand\" forces\n");
printf("               this program to conform to that convention producing one image per slice.\n\n");
printf("    -nopeaks   Do not produce the .peaks.bfloat images.\n\n");             
}

