% coltdrmtx.m - program to compute the matrix that reconstructs the
% columns of the k-space image based on the time-domain
% reconstruction method.

addpath /homes/4/greve/sg2/dale-matlab/utils; % for smoother

outfile = '/space/greve/2/users/greve/dng072203/RcolTDR50.mask.svd.mat';

% EPI parameters
TE = 50;              % Echo time in ms
epiechospacing = 470; % us
delsamp = 30;         % us
tDwell = 3.2;         % us

% Flag indicating reversal in PE direction
if(TE==50) perev = 1;
else       perev = 0;
end

% Free induction decay (FID) map
fiddir = '/space/greve/2/users/greve/dng072203/fid1/mgh';
fidecho1ped = 1810;   % PED of first echo (us)
fidechospacing = 820; % Time between FID echoes (Actually use double)
nfidechoes = 99; % But we'll only use half (odd echoes)
rthreshfid = 0.20;
%rthreshfid = 0.0;
fwhm = 0;

% SVD regularization percentage
svdregpct = 90;
tikregfact = 0.5; % Tikhonov regularization

% Applies to both EPI and FID - should just get this from data
nrows = 64;
ncols = 128;
nslices = 35;
nv = nrows*ncols;

evenrows = [2:2:nrows];
oddrows  = [1:2:nrows];

% Start Timer
tic;

% Only use odd FID echoes
fidechospacing_odd = 2*fidechospacing;
nfidechoes_odd = length([1:2:nfidechoes]);

% Compute the Ideal col and row DFT reconstruction matrices
Fcol = fast_dftmtx(nrows);
Rcol = inv(Fcol);
FcolEPI = Fcol;
if(perev) FcolEPI = flipud(FcolEPI); end
Frow = fast_dftmtx(ncols);
Rrow = transpose(inv(Frow));
D = zeros(nrows,nrows,ncols,nslices);
D0 = zeros(nrows,ncols,nslices);
RcolTDR = zeros(nrows,nrows,ncols,nslices);
RcolTDRcond = zeros(ncols,nslices);
RcolTDRdim = zeros(ncols,nslices);

% Get the PED of each sample in the EPI
pedmat = pedmatrix2(TE*1000,epiechospacing,delsamp,tDwell,nrows,ncols,perev);
pedmat(evenrows,:) = fliplr(pedmat(evenrows,:));

% Comute the FID echo indices corresponding to the times at which
% the EPI k-space samples were taken. Weights are for lin interp.
[indA, indB, wA, wB] = fidinterp(pedmat,fidecho1ped,fidechospacing_odd,nfidechoes_odd);

for sliceno = 1:nslices
  fprintf('sliceno = %d (%g) ----------------\n',sliceno,toc);
  fid = zeros(nrows,ncols,nfidechoes_odd);

  nthefid = 1;
  for efid = 1:2:nfidechoes
    %fprintf('echo = %d, toc = %g\n',e,toc);
    
    % Load the real part of this FID echo 
    mghname = sprintf('%s/echor%d.mgh',fiddir,efid-1);
    kfidr = load_mgh(mghname);
    if(isempty(kfidr)) return; end
    kfidr = squeeze(kfidr(:,:,sliceno))'; % transpose for row major
    
    % Load the imagniary part of this FID echo 
    mghname = sprintf('%s/echoi%d.mgh',fiddir,efid-1);
    kfidi = load_mgh(mghname);
    if(isempty(kfidi)) return; end
    kfidi = squeeze(kfidi(:,:,sliceno))'; % transpose for row major
    
    % Compute the complex
    kfid = kfidr + i*kfidi;
    
    % Recon the FID image
    fidimg = Rcol * kfid * Rrow;

    if(fwhm > 0)
      fidimg = smooth2d(fidimg,fwhm,fwhm);
    end

    if(perev) fidimg = flipud(fidimg); end
    
    fid(:,:,nthefid) = fidimg;
    
    nthefid = nthefid + 1;
  end % finished loading all FID echoes

  if(rthreshfid > 0)
    % set decay map to 1 for subthresh voxels
    fidimg1 = abs(fid(:,:,1));
    fidimg1mn = mean(reshape1d(fidimg1));
    indsubthresh = find(fidimg1 < rthreshfid*fidimg1mn);
    nsubthresh = length(indsubthresh);
    fprintf('Found %d (%g %%) voxels below thresh\n',...
	    nsubthresh,100*nsubthresh/nv);
    fid = reshape(fid,[nv nfidechoes_odd]);
    fid(indsubthresh,:) = 1;
    fid = reshape(fid,[nrows ncols nfidechoes_odd]);
  end
  D0(:,:,sliceno) = fid(:,:,1);
  
  for imgcol = 1:ncols

    Dcol  = zeros(nrows,nrows);
    for imgrow = 1:nrows
      fid1A = squeeze(fid(imgrow,imgcol,indA(:,imgcol)));
      fid1B = squeeze(fid(imgrow,imgcol,indB(:,imgcol)));
      d = wA(:,imgcol) .* fid1A + wB(:,imgcol) .* fid1B;
      Dcol(:,imgrow) = d;
    end
    D(:,:,imgcol,sliceno) = Dcol; % save unnormalized

    Dcol0 = transpose(D0(:,imgcol,sliceno));
    Dcol = Dcol ./ repmat(abs(Dcol0),[nrows 1]);
    %Dcol = Dcol ./ repmat(abs(Dcol(1,:)),[nrows 1]);

    FcolTDR = Dcol .* FcolEPI;

    %FcolTDRdim = 0;
    [FcolTDRreg FcolTDRdim] = fast_svdregpct(FcolTDR,svdregpct);
    RcolTDR = inv(FcolTDRreg);

    %tmp = FcolTDR'*FcolTDR;
    %RcolTDR = inv( tmp + tikregfact*eye(size(tmp))*mean(diag(tmp)) )*FcolTDR';

    RcolTDRmag = sqrt(abs(sum(RcolTDR.*conj(RcolTDR),2)));
    RcolTDR = RcolTDR./repmat(RcolTDRmag,[1 nrows]);
    RTDR(:,:,imgcol,sliceno) = RcolTDR;
    RTDRcond(imgcol,sliceno) = cond(RcolTDR);
    RTDRdim(imgcol,sliceno) = FcolTDRdim;
  
  end

end % slice

save(outfile,'D','RTDR','RTDRcond','RTDRdim','svdregpct',...
     'TE','epiechospacing','delsamp','tDwell','perev','fiddir',...
     'fidecho1ped','fidechospacing','nfidechoes','pedmat',...
     'indA','indB','wA','wB','nrows','ncols','nslices',...
     'rthreshfid','D0','fwhm','tikregfact');


toc