% fast_svdfilter.m

% extregname
% mergeextreg
% mergedextreg
% filter option

tic

if(0)
  fsd = '/home/greve/sg1/nh-rest/bold';
  analysis = 'onoff60s';
  runidlist = strvcat('006','007','008','009');
  fsd = '/space/greve/1/users/greve/wagner/JL/bold';
  analysis = 'onoff';
  runidlist = strvcat('006','007','008','009');
  extregrstem = sprintf('%s-xreg',analysis);
  funcrstem = 'fmcsm5';
  maskrstem = 'brain';
  dimfrac   = 1;
  sigthresh = 3;
end

fprintf('\n\n');
fprintf('fast_svdfilter\n');
fprintf('%s\n',pwd);
fprintf('fsd = %s\n',fsd);
fprintf('analysis = %s\n',analysis);
fprintf('mask = %s\n',maskrstem);
fprintf('func      = %s\n',funcrstem);
fprintf('filfunc   = %s\n',filfuncrstem);
fprintf('extreg    = %s\n',extregrstem);
fprintf('dimfrac   = %g\n',dimfrac);
fprintf('sigthresh = %g\n',sigthresh);
fprintf('\n\n');

%--------------------------------------------------%
pthresh = 10.^(-sigthresh);

maskastem = sprintf('%s/masks/%s',fsd,maskrstem);
mask = fmri_ldbvolume(maskastem);
if(isempty(mask))
  fprintf('ERROR: could not load %s\n',maskastem);
  return; quit;
end
indmask = find(mask);
nmask = length(find(mask));
if(nmask == 0)
  fprintf('ERROR: could not find any voxels in mask\n');
  return; quit;
end
fprintf('INFO: found %d voxels in mask\n',nmask);
  

fprintf('\n\n');
nruns = size(runidlist,1);
for nthrun = 1:nruns

  runid = deblank(runidlist(nthrun,:));
  fprintf('run %s   (%g) -----------------\n',runid,toc);

  funcastem = sprintf('%s/%s/%s',fsd,runid,funcrstem);
  xmatfile  = sprintf('%s/%s/%s.mat',fsd,runid,analysis);

  XX = load(xmatfile);
  if(isempty(XX))
    fprintf('ERROR: could not load %s\n',xmatfile);
    return; quit;
  end
  X0 = XX.Xfinal;
  X = X0;
  nf = size(X,1);
  Ntask = XX.Nnnc*XX.Navgs_per_cond;
  Nbeta = size(X,2);
  C = [eye(Ntask) zeros(Ntask,Nbeta-Ntask)];
  dof = size(X,1)-size(X,2);
  B = inv(X'*X)*X';
  R = eye(nf)-X*B;
  
  fprintf('Loading functionals (%g)\n',toc);
  y = fmri_ldbvolume(funcastem);
  if(isempty(y))
    fprintf('ERROR: could not load %s\n',funcastem);
    return; quit;
  end
  [ns nr nc nf] = size(y);
  nv = ns*nr*nc;
  
  if(nf ~= size(X,1))
    fprintf('ERROR: mismatch in the number of frames\n');
    return; quit;
  end
  if(nv ~= prod(size(mask)))
    fprintf('ERROR: mismatch in the number of voxels\n');
    return; quit;
  end
  y = reshape(y,[nv nf])';
  y0 = y;
  y = y(:,indmask);
  ymean = mean(reshape1d(y));
  
  fprintf('Analyzing functionals to get exclusion mask (%g)\n',toc);
  beta = B*y;
  r = y-X*beta;
  rvar = sum(r.^2)/dof;
  [F p g] = fast_fratio(beta,X,rvar,C);
  [pdf alpha nx fpr] = ComputePDF(p,0,1,.001);
  
  indexclude = find(p < pthresh);
  indinclude = find(p >= pthresh);
  nexclude = length(indexclude);
  ninclude = length(indinclude);
  fprintf('Excluding %d (%g%%) voxels as active (th=%g)\n',...
	  nexclude,100*nexclude/nmask,sigthresh);
  fprintf('Continuing with %d voxels\n',ninclude);
  y = y(:,indinclude);
  
  fprintf('Removing mean from functionals (%g)\n',toc);
  ymn = mean(y);
  y = y - repmat(ymn,[nf 1]);
  
  fprintf('Computing My (%g)\n',toc);
  My = y*y'; %'
  
  [u s blah] = svd(My);
  ds = diag(s);
  %plot(100*ds/sum(ds));
  
  [dim0 pvs pvsw] = fast_estdimsvd(s);
  dim = ceil(dim0*dimfrac);
  fprintf('run = %s  dim0 = %d (%g), dim = %d (%g)\n',...
	  runid,dim0,sum(pvs(1:dim0)),dim,sum(pvs(1:dim)));
  
  % Only keep the first dim components of u %
  u = u(:,1:dim);

  % Test u for signal %
  beta = B*u;
  uhat = X0*beta;
  r = u - uhat;
  rvar = sum(r.^2)/dof;
  [F p] = fast_fratio(beta,X,rvar,C);
  indpexcl = find(p < pthresh);
  indpincl = find(p >= pthresh);
  fprintf('Excluding %d eigenvectors\n',length(indpexcl));
  fprintf('Continuing with %d \n',length(indpincl));
  u = u(:,indpincl);
  
  extregstem = sprintf('%s/%s/%s',fsd,runid,extregrstem);
  clear tmp;
  tmp(1,:,:) = u';
  fmri_svbvolume(tmp,extregstem);
  fprintf('Saving extreg to %s\n',extregstem);

  if(~isempty(filfuncrstem))
    fprintf('Filtering functional\n');
    M = eye(nf) - u*u';
    yfil = M*y0;
    yfil = reshape(yfil',[ns nr nc nf]);
    filfuncstem = sprintf('%s/%s/%s',fsd,runid,filfuncrstem);
    fprintf('Saving to %s\n',filfuncstem);
    fmri_svbvolume(yfil,filfuncstem);
    meanvalfile = sprintf('%s.meanval',filfuncstem);
    fid = fopen(meanvalfile,'w');
    fprintf(fid,'%f\n',ymean);
    fclose(fid);
  end
  
  
  if(0)
    X = [X0 u];
    Ntask = XX.Nnnc*XX.Navgs_per_cond;
    Nbeta = size(X,2);
    C = zeros(Ntask,Nbeta);
    C(1:Ntask,1:Ntask) = 1;
    dof = size(X,1)-size(X,2);
    B = inv(X'*X)*X';
    beta = B*y0(:,indmask);
    r = y0(:,indmask)-X*beta;
    rvar = sum(r.^2)/dof;
    [F p g] = fast_fratio(beta,X,rvar,C);
    [pdf2 alpha2 nx fpr2] = ComputePDF(p,0,1,.001);
    figure(nthrun);
    loglog(alpha,alpha,alpha,fpr,alpha2,fpr2);
    legend('ideal','none','svdfil');
    title(sprintf('Run %s',runid));
    drawnow;
  end

  fprintf('\n\n');
  
end

toc









