function err = fast_write_wfile(fname, w, v)
% err = fast_write_wfile(fname, w, v)
% 
% writes a vector into a binary 'w' file
%  fname - name of file to write to
%  w     - vector of values to be written
%  v     - 0-based vertex numbers 
%          (assumes 0 to N-1 if not present or empty).

if(nargin ~= 2 & nargin ~= 3)
  fprintf('USAGE: [w] = fast_write_wfile(fname, w, <v>) \n');
  return;
end

vnum = length(w) ;

if(exist('v') ~= 1) v = []; end
if(isempty(v)) v = [0:vnum-1]; end

% open it as a big-endian file
fid = fopen(fname, 'wb', 'b') ;

fwrite(fid, 0, 'int16') ;
fwrite3(fid, vnum) ;
for i=1:vnum
  fwrite3(fid, v(i)) ;
  fwrite(fid,  w(i), 'float') ;
end

fclose(fid) ;

return
