function img = tdr_recon_fourier(kimg,Rrow,PERev,PEShift)
% img = tdr_recon_fourier(kimg,Rrow,PERev,<PEShift>)
%
% Does a "standard" fourier recon. The rows are reconned by
% applying Rrow. The columns are reconned using the inverse
% fft. 
% 
% PEShift is the number of voxels to shift in the phase-encode 
% direction (can be fractional). Positive values of
% PEShift will shift the object down (?).
%
% $Id$

if(nargin ~= 3 & nargin ~= 4)
  fprintf('img = tdr_recon_fourier(kimg,Rrow,PERev,<PEShift>)\n');
  return;
end

if(exist('PEShift') ~= 1) PEShift = []; end

[nrows ncols nslices nframes] = size(kimg);
evenrows = [2:2:nrows];
Fcol = fast_dftmtx(nrows);
Rcol = inv(Fcol);

if(~isempty(PEShift))
  ph = 2*pi * [0:nrows-1]'/nrows;
  ph = ph - ph(nrows/2+1); 
  vdelta = exp(-i*ph*PEShift);
  kimg = kimg .* repmat(vdelta, [1 ncols nslices nframes]);
end

img = zeros(size(kimg));
for frame = 1:nframes
  for slice = 1:nslices
    kslice = kimg(:,:,slice,frame);
    kslice(evenrows,:) = fliplr(kslice(evenrows,:));
    ksliceB = tdr_deghost(kslice,Rrow,PERev);
    %img(:,:,slice,frame) = abs(Rcol*ksliceB); % fft faster
    img(:,:,slice,frame) = abs(fftshift(ifft(fftshift(ksliceB,1),[],1),1));
  end
end

return;












