# tkUtils.tcl (tku)

# $Id: tkUtils.tcl,v 1.5 2004/02/09 17:17:38 kteich Exp $

# tkuMakeMenu isMenuButton "Menu Name" {item...}
# item = { command   "Item Name" command                [group_name] }
# item = { radio     "Item Name" command variable value [group_name] }
# item = { check     "Item Name" command variable       [group_name] }
# item = { cascade   "Item Name" {item...}              [group_name] }
# item = { separator }

set kLabelFont -*-lucida-bold-r-normal-*-14-*-*-*-*-*-*-*
set kNormalFont -*-lucida-medium-r-normal-*-12-*-*-*-*-*-*-*

proc tkuLabelFont {} {
    global kLabelFont
    return $kLabelFont
}

proc tkuNormalFont {} {
    global kNormalFont
    return $kNormalFont
}

# tkuMakeActiveLabel
# -variable : variable for the value (required)
# -label : label preceding the value
# -width : width of the variable entry
proc tkuMakeActiveLabel { ifwTop args } {
    global kLabelFont

    # set default arguments for all fields
    set aArgs(-label) ""
    set aArgs(-width) 0
    set aArgs(-height) 0
    set aArgs(-font) $kLabelFont
    set aArgs(-labelwidth) 0
    set aArgs(-labelheight) 0

    # Set arg items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-variable} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeActiveLabel: no $arg specified"
	    return
	}
    }

    frame $ifwTop

    if { $aArgs(-label) != "" } {
	label $ifwTop.lw \
		-font $aArgs(-font) \
		-text $aArgs(-label) \
		-width $aArgs(-labelwidth) \
		-height $aArgs(-labelheight) \
		-anchor e

	pack $ifwTop.lw \
		-side left \
		-expand yes \
		-fill x \
		-anchor e \
		-padx 5
    }

#    entry $ifwTop.ew \
#      -textvariable $aArgs(-variable) \
#      -width $aArgs(-width) \
#      -state disabled \
#      -relief flat

    label $ifwTop.ew \
	    -textvariable $aArgs(-variable) \
	    -font $aArgs(-font) \
	    -width $aArgs(-width) \
	    -height $aArgs(-height) \
	    -relief flat \
	    -anchor w

    pack $ifwTop.ew \
	    -side left \
	    -expand yes \
	    -fill x \
	    -anchor w
}

# tkuMakeNormalLabel
# -label : text for the label (reqd)
# -width : hard width of the label or 0 for flexible
# -wrap : the length at which to wrap text
# -font : font to use
# -anchor : side to anchor (n, e, se, etc)
proc tkuMakeNormalLabel { ifwTop args } {
    global kNormalFont

    # set default arguments for all fields
    set aArgs(-wrap) 0
    set aArgs(-width) 0
    set aArgs(-font) $kNormalFont
    set aArgs(-justify) left
    set aArgs(-anchor) w

    # Set arg items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-label} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeNormalLabel: no $arg specified"
	    return
	}
    }

    frame $ifwTop
    
    label $ifwTop.lw \
	-font $aArgs(-font) \
	-text $aArgs(-label) \
	-width $aArgs(-width) \
	-justify left \
	-anchor $aArgs(-anchor) \
	-wraplength $aArgs(-wrap)

    if { $aArgs(-wrap) != 0 } {
	$ifwTop.lw configure -wraplength $aArgs(-wrap)
    }

    pack $ifwTop.lw \
      -side left \
      -anchor w
}

proc tkuMakeEntry { ifwTop args } {
    global kLabelFont
    
    # set default arguments for all fields
    set aArgs(-label) ""
    set aArgs(-width) 0
    set aArgs(-font) $kLabelFont
    set aArgs(-labelwidth) 0
    set aArgs(-command) ""

    # Set arg items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-variable} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeEntry: no $arg specified"
	    return
	}
    }

    frame $ifwTop

    if { $aArgs(-label) != "" } {
	
	label $ifwTop.lwLabel \
		-width $aArgs(-labelwidth) \
		-text $aArgs(-label) \
		-font $aArgs(-font)
	
	pack $ifwTop.lwLabel \
		-side left \
		-anchor w
    }

    entry $ifwTop.ewEntry \
      -textvariable $aArgs(-variable) \
      -width $aArgs(-width)
    
    pack $ifwTop.ewEntry \
      -side right \
      -anchor e \
      -expand yes \
      -fill x

    if { $aArgs(-command) != "" } {
	bind $ifwTop.ewEntry <Return> "$aArgs(-command)"
    }

}


# tkuMakeMenu
# -menu : location of the new menu button widget (required)
# -label : the name of the menu button
# -items : list of items
proc tkuMakeMenu { args } {

    # Set menu items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-menu -label -items} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeMenu: no $arg specified"
	    return
	}
    }

    # for every letter in menu name..
    # if this letter is not in the underline list...
    # add the letter to the list.
    # underline this index.
    # if no letters left, underline nothing.
    
    menubutton $aArgs(-menu) \
	-text $aArgs(-label) \
	-menu $aArgs(-menu).mw
    #      -underline $nUnderline
    
    # start an underline list for this menu
    
    tkuAddMenuItemsToMenu $aArgs(-menu).mw $aArgs(-items)
}

proc tkuAddMenuItemsToMenu { isMenu ilMenuItems } {
    
    menu $isMenu
    set nItemNum 1
    
    foreach lItem $ilMenuItems {
	
	set sType [lindex $lItem 0]
	
	set bProcessed 0
	
	if { [string compare $sType "command"] == 0 } {
	    
	    # for every letter in item name..
	    # if this letter is not in the local underline list...
	    # add the letter to the list.
	    # underline this index.
	    # if no letters left, underline nothing.
	    
	    $isMenu add command \
		-label [lindex $lItem 1] \
		-command [lindex $lItem 2]
	    #              -underline $nUnderline
	    
	    set sGroupName [lindex $lItem 3]
	    if { [string compare $sGroupName ""] != 0 } {
		tkuAddItemToMenuGroup $sGroupName $isMenu $nItemNum
	    }
	    
	    set bProcessed 1
	}
	if { [string compare $sType "radio" ] == 0 } {
	    $isMenu add radio \
		-label [lindex $lItem 1] \
		-command [lindex $lItem 2] \
		-variable [lindex $lItem 3] \
		-value [lindex $lItem 4]
	    
	    set sGroupName [lindex $lItem 5]
	    if { [string compare $sGroupName ""] != 0 } {
		tkuAddItemToMenuGroup $sGroupName $isMenu $nItemNum
	    }
	    
	    set bProcessed 1
	}
	if { [string compare $sType "check" ] == 0 } {
	    $isMenu add check \
		-label [lindex $lItem 1] \
		-command [lindex $lItem 2] \
		-variable [lindex $lItem 3]
	    
	    set sGroupName [lindex $lItem 4]
	    if { [string compare $sGroupName ""] != 0 } {
		tkuAddItemToMenuGroup $sGroupName $isMenu $nItemNum
	    }
	    
	    set bProcessed 1
	}
	if { [string compare $sType "separator"] == 0 } {
	    $isMenu add separator
	    set bProcessed 1
	}
	if { [string compare $sType "cascade"] == 0 } {
	    $isMenu add cascade \
		-label [lindex $lItem 1] \
		-menu $isMenu.cmw$nItemNum
	    
	    set lCascadeItems [lindex $lItem 2]
	    tkuAddMenuItemsToMenu $isMenu.cmw$nItemNum $lCascadeItems
	    
	    set sGroupName [lindex $lItem 3]
	    if { [string compare $sGroupName ""] != 0 } {
		tkuAddItemToMenuGroup $sGroupName $isMenu $nItemNum
	    }
	    set bProcessed 1
	}
	
	if { $bProcessed == 0 } {
	    puts "Error!!!!! $sType not recognized"
	}
	
	if { $bProcessed == 1 } {
	    incr nItemNum
	}
    }
}

proc tkuAddItemToMenuGroup { isGroupName ifwMenuObject inMenuItemNum } {
    
    global glMenuGroups
    
    # add this menu / item pair to the list
    lappend glMenuGroups($isGroupName) "$ifwMenuObject $inMenuItemNum"
}

proc tkuSetMenuItemGroupStatus { isGroupName ibEnable } {
    
    global glMenuGroups
    
    # for each menu / item pair in the list
    foreach lMenuItemPair $glMenuGroups($isGroupName) {
	
	# first item is a menu button
	set mbwMenu [lindex $lMenuItemPair 0]
	
	# second item is a list of items
	set nMenuItem [lindex $lMenuItemPair 1]
	
	if { $ibEnable == 0 } {
	    if { [catch {$mbwMenu entryconfigure $nMenuItem -state disabled} sResult] } {
		set sType [$mbwMenu type $nMenuItem]
		puts "error, $isGroupName: $mbwMenu $nMenuItem $sType\n\t$sResult"
		
	    }
	} else {
	    if { [catch {$mbwMenu entryconfigure $nMenuItem -state normal} sResult] } {
		puts "error, $isGroupName: $mbwMenu $nMenuItem\n\t$sResult"
	    }
	}
    }
}


# tkuMakeCheckboxes
# -orientation : orientation of checkboxes (h, v)
# -font : overall font
# -labelwidth : overall label width
# -labelheight : overall label height
# -checkboxes : list of checkbox data (required)
#    -type : type of checkbox label (text, image)
#    -label : text label
#    -image : image label
#    -variable : variable to set (required)
#    -font : font for this checkbox label
#    -command : command to run when clicked
proc tkuMakeCheckboxes { ifwTop args } {

    # set default arguments for all fields
    set aArgs(-orientation) v
    set aArgs(-font) [tkuLabelFont]
    set aArgs(-labelwidth) 0
    set aArgs(-labelheight) 0

    # Set arg items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-checkboxes} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeCheckboxes: no $arg specified"
	    return
	}
    }

    frame $ifwTop
    
    # for each checkbox...
    set nCheckbox 0
    foreach lCheckbox $aArgs(-checkboxes) {
	
	# Set arg items and make sure we have the ones we require,
	set aCheckbox(-type) text
	set aCheckbox(-label) ""
	set aCheckbox(-font) $aArgs(-font)
	set aCheckbox(-labelwidth) $aArgs(-labelwidth)
	set aCheckbox(-labelheight) $aArgs(-labelheight)
	set aCheckbox(-command) ""
	array set aCheckbox $lCheckbox
	foreach arg {-variable} {
	    if {![info exists aCheckbox($arg)]} {
    	     puts "tkuMakeCheckboxes: no $arg specified in checkbox $nCheckbox"
		return
	    }
	}

	# make names for the checkbox and the label.
	set cbw $ifwTop.cb$nCheckbox
	set lw  $ifwTop.lw$nCheckbox
	
	# text or image?
	switch $aCheckbox(-type) {
	    
	    text {
		
		# text. make a normal checkbox and label.
		checkbutton $cbw \
		    -variable $aCheckbox(-variable) \
		    -command $aCheckbox(-command)
		label $lw \
		    -font $aCheckbox(-font) \
		    -text $aCheckbox(-label)
		
		# if horizontal, pack all in the same row. if vertical.
		# pack the checkbox, than the label, in the
		# same row as the number of this checkbox.
		switch $aArgs(-orientation) {
		    h - x { 
			grid $cbw -column [expr 2 * $nCheckbox] -row 0
			grid $lw -column [expr 1 + [expr 2 *$nCheckbox]] \
			    -row 0 -sticky w
		    }
		    v - y { 
			grid $cbw -column 0 -row $nCheckbox
			grid $lw -column 1 -row $nCheckbox -sticky w
		    }
		}
	    }
	    
	    image { 
		# image. create a checkbox with an image label. no text label.
		checkbutton $cbw \
		    -image $aCheckbox(-image) \
		    -variable $aCheckbox(-variable) \
		    -command $aCheckbox(-command) \
		    -indicatoron false \
		    -selectcolor gray
		
		# if horizontal, pack in increasing columns. if vertical,
		# pack in increasing rows.
		switch $isDirection {
		    h - x { 
			grid $cbw -column $nCheckbox -row 0
		    }
		    v - y { 
			grid $cbw -column 0 -row $nCheckbox
		    }
		}
	    }
	    
	    default { continue }
	}
	
	incr nCheckbox
    }
    
    grid columnconfigure $ifwTop 0 -weight 0
    grid columnconfigure $ifwTop 1 -weight 1
}

# tkuMakeSliders
# -font : overall font
# -sliders : list of slider data (required)
#    -orientation : orientation of this slider (h, v)
#    -label : label to left of slider
#    -postlabel : label to right of slider
#    -font : font for this slider
#    -variable  : variable to set (required)
#    -min : min value
#    -max : max value
#    -resolution : numerical resolution of slider
#    -length : length of slider
#    -command : command to call when changed
#    -entry : whether to include a text entry
proc tkuMakeSliders { ifwTop args } {

    # set default arguments for all fields
    set aArgs(-font) [tkuLabelFont]

    # Set arg items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-sliders} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeSliders: no $arg specified"
	    return
	}
    }

    frame $ifwTop
    
    # for each slider...
    set nSlider 0
    foreach lSlider $aArgs(-sliders) {
	
	# Set arg items and make sure we have the ones we require,
	set aSlider(-orientation) h
	set aSlider(-label) ""
	set aSlider(-postlabel) ""
	set aSlider(-font) $aArgs(-font)
	set aSlider(-min) 0
	set aSlider(-max) 10
	set aSlider(-resolution) 1.0
	set aSlider(-length) 100
	set aSlider(-entry) 0
	set aSlider(-command) ""
	array set aSlider $lSlider
	foreach arg {-variable} {
	    if {![info exists aSlider($arg)]} {
    	     puts "tkuMakeSlider: no $arg specified in slider $nSlider"
		return
	    }
	}

	tkuMakeNormalLabel $ifwTop.lw$nSlider -label $aSlider(-label)
	tkuMakeNormalLabel $ifwTop.lwPost$nSlider -label $aSlider(-postlabel)

	scale $ifwTop.sw$nSlider \
	    -orient $aSlider(-orientation) \
	    -variable $aSlider(-variable) \
	    -from $aSlider(-min) \
	    -to $aSlider(-max) \
	    -length $aSlider(-length) \
	    -resolution $aSlider(-resolution) \
	    -showvalue false
	bind $ifwTop.sw$nSlider <ButtonRelease> $aSlider(-command)
	bind $ifwTop.sw$nSlider <B1-Motion> $aSlider(-command)

	if { $aSlider(-entry) } {
	    entry $ifwTop.ew$nSlider \
		-textvariable $aSlider(-variable) \
		-width 4 \
		-selectbackground green \
		-insertbackground black
	    bind $ifwTop.ew$nSlider <Return> $aSlider(-command)
	}

	# if horizontal, pack all in the same row. if vertical.
	# pack the checkbox, than the label, in the
	# same row as the number of this checkbox.
	grid $ifwTop.lw$nSlider     -column 0 -row $nSlider -sticky w
	grid $ifwTop.sw$nSlider     -column 1 -row $nSlider -sticky ew
	if { $aSlider(-entry) } {
	    grid $ifwTop.ew$nSlider -column 2 -row $nSlider -sticky w
	    grid $ifwTop.lwPost$nSlider -column 3 -row $nSlider -sticky w
	} else {
	    grid $ifwTop.lwPost$nSlider -column 2 -row $nSlider -sticky w
	}		
	
	incr nSlider
    }

    grid columnconfigure $ifwTop 0 -weight 0
    grid columnconfigure $ifwTop 1 -weight 1
    grid columnconfigure $ifwTop 2 -weight 0
    grid columnconfigure $ifwTop 3 -weight 0
}

# Based on new min and max, will change -from and -to of all sliders
# in this group and recalculate a decent resolution.
proc tkuUpdateSlidersRange { ifwTop iMin iMax } {

    # See what our value range is and set a good resolution.
    set diff [expr $iMax - $iMin]
    if { $diff > 1000 } {
	set newResolution 10
    } elseif { $diff > 100 } {
	set newResolution 1
    } elseif { $diff > 10 } {
	set newResolution .1
    } elseif { $diff > 1 } {
	set newResolution .001
    } elseif { $diff > 0.1 } {
	set newResolution .0001
    } elseif { $diff > 0.01 } {
	set newResolution .00001
    } elseif { $diff > 0.001 } {
	set newResolution .000001
    } elseif { $diff > 0.0001 } {
	set newResolution .0000001
    } elseif { $diff > 0.00001 } {
	set newResolution .00000001
    } elseif { $diff > 0.000001 } {
	set newResolution .000000001
    } else {
	set newResolution .0000000001
    }

    set nSlider 0
    set err 0
    while { $err == 0 } {

	set err [catch { 
	    $ifwTop.sw$nSlider config -from $iMin -to $iMax \
		-resolution $newResolution}]

	incr nSlider
    }
}

# tkuMakeToolbar
# -allowzero : whether or not the toolbar can have no buttons on
# -radio : whether or not the toolbar can only have one button on
# -variable : variable to set (required)
# -command : command to call when value changes
# -font : font for all buttons
# -buttons : list of button arrays
#    -type : type of button (text,image)
#    -name : name and value of this button
#    -label : text label for text button
#    -image : image for image button
#    -font : font for this button
proc tkuMakeToolbar { ifwTop args } {
    
    # set default arguments for all fields
    set aArgs(-allowzero) false
    set aArgs(-radio) true
    set aArgs(-command) ""
    set aArgs(-font) [tkuNormalFont]

    # Set arg items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-variable -buttons} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeToolbar: no $arg specified"
	    return
	}
    }

    frame $ifwTop
    
    tixSelect $ifwTop.tbw \
	-allowzero $aArgs(-allowzero) \
	-radio $aArgs(-radio) \
	-command $aArgs(-command) \
	-disablecallback true
    
    tkuEnableLater $ifwTop.tbw
    
    foreach lButton $aArgs(-buttons) {
	
	# set default arguments for all fields
	set aButton(-type) text
	set aButton(-label) ""
	set aButton(-font) $aArgs(-font)
	array set aButton $lButton
	foreach arg {-name} {
	    if {![info exists aButton($arg)]} {
		puts "tkuMakeToolbar: no $arg specified for toolbar button"
		return
	    }
	}

	switch $aButton(-type) {

	    text {
		$ifwTop.tbw  \
		    add $aButton(-name) \
		    -text $aButton(-label) \
		    -font $aButton(-font)
	    } 
	    image {
		$ifwTop.tbw \
		    add $aButton(-name) \
		    -image $aButton(-image)
	    }
	    default {
		puts "didn't handle type $aButton(-type)"
	    }
	} 
    }
    
    $ifwTop.tbw config -variable $aArgs(-variable)
    
    pack $ifwTop.tbw
}

proc tkuEnableLater { ifwWidget } {
    global glDisabledWidgets
    lappend glDisabledWidgets $ifwWidget
}

proc tkuFinish { } {
    global glDisabledWidgets
    foreach widget $glDisabledWidgets {
	$widget config -disablecallback false
    }
}

# replaces the percent symbols in a string with a substitution string:
# i.e. tkuDoSubPercent { %s1 "Hello %s1" "World" }
# returns "Hello World"
proc tkuDoSubPercent { isPercent isString isSubstitution } {
    if {![string match %* $isPercent]} {
	return $isString;
    }
    regsub -all {\\|&} $isSubstitution {\\\0} isSubstitution
    regsub -all $isPercent $isString $isSubstitution sResult
    return $sResult
}

# tkuDoSubPercent
# -title : dlog title
# -okCmd : cmd to execute on OK
# -cancel : cmd to execute on cancel
# -promptN : prompt for item N (required to make this item active)
# -noteN : note for item N
# -typeN : type for item N (file,dir,text,checkbutton)
# -defaultfuncN : func to return default dir for item N
# -defaultdirN : default dir for item N
# -shortcutdirs :  shortcut list for all file and dir itmes
# -shortcutdirsN : shortcut list for item N (will override above)
# -defaultvalueN : initial value for item N

# USAGE:
# tkuDoFileDlog -title "Save File" \
#  -prompt1 "Save File As:" \
#  -note1 "The file name to write" \
#  -defaultfunc1 [list GetDefaultLocation SaveFile] \
#  -shortcutdirs {/usr /usr/bin ~/documents} \
#  -okCmd {SaveFile %s1; \
#          SetDefaultLocation SaveFile %s1}
proc tkuDoFileDlog { args } {

    global gDialog
    global sFileName1 sFileName2 sFileName3 sFileName4 sFileName5

    set lFields {1 2 3 4 5}
    set knWidth 400
    
    # set default arguments for all fields
    set aArgs(-title) "No title"
    set aArgs(-okCmd) "puts \$sFileName"
    set aArgs(-cancelCmd) ""
    foreach nField $lFields {
	set aArgs(-prompt$nField) ""
	set aArgs(-type$nField) "file"
	set aArgs(-note$nField) ""
	set sFileName$nField ""
    }
    
    # allow passed options to override defaults
    array set aArgs $args

    # dialog name. make it the name of the title, subbing dashes for spaces.
    regsub -all { } .wwDlogFile$aArgs(-title) {-} wwDialog

    # do percent substitutions in ok command for %s1 thru %s5
    foreach nField $lFields {
	set aArgs(-okCmd) \
	    [tkuDoSubPercent %s$nField $aArgs(-okCmd) \$sFileName$nField]
    }
    
    # if we can bring up the dialog
    if { [tkuCreateDialog $wwDialog "$aArgs(-title)" {-borderwidth 10}] } {
	
	# for each field...
	foreach nField $lFields {
	    
	    # create a variable for this prompt. even if we don't use this
	    # field, we'll need it later (ugh)
	    set fwPrompt$nField  $wwDialog.fwPrompt$nField
	    
	    # if they didn't enter a prompt, skip this field
	    if { [string match "$aArgs(-prompt$nField)" ""] == 1 } {
		continue
	    }
	    
	    set sDefaultDirOption ""
	    set sDefaultDirValue ""
	    if { [info exists aArgs(-defaultfunc$nField)] } {
		set sDefaultDirOption -defaultfunc
		set sDefaultDirValue $aArgs(-defaultfunc$nField)
	    }
	    if { [info exists aArgs(-defaultdir$nField)] } {
		set sDefaultDirOption -defaultdir
		set sDefaultDirValue $aArgs(-defaultdir$nField)
	    }
	    set sShortcutDirsOption ""
	    set sShortcutDirsValue ""
	    if { [info exists aArgs(-shortcutdirs$nField)] } {
		set sShortcutDirsOption -shortcutdirs
		set sShortcutDirsValue $aArgs(-shortcutdirs$nField)
	    }
	    if { [info exists aArgs(-shortcutdirs)] } {
		set sShortcutDirsOption -shortcutdirs
		set sShortcutDirsValue $aArgs(-shortcutdirs)
	    }

	    if { [info exists aArgs(-defaultvalue$nField)] } {
		set sFileName$nField $aArgs(-defaultvalue$nField)
	    }
	    
	    # switch on the type for this field and create the approriate
	    # selecter widget. bind it to sFileName[1..5]
	    switch $aArgs(-type$nField) {
		file { 
		    tkuMakeFileSelector [set fwPrompt$nField] \
			-text "$aArgs(-prompt$nField)" \
			-variable sFileName$nField \
			$sDefaultDirOption $sDefaultDirValue \
			$sShortcutDirsOption $sShortcutDirsValue
		}
		dir { 
		    tkuMakeDirectorySelector [set fwPrompt$nField] \
			-text "$aArgs(-prompt$nField)" \
			-variable sFileName$nField \
			$sDefaultDirOption $sDefaultDirValue \
			$sShortcutDirsOption $sShortcutDirsValue
		}
		text { 
		    tkuMakeEntry [set fwPrompt$nField] \
			-label "$aArgs(-prompt$nField)" \
			-variable sFileName$nField 
		}
		checkbox { 
		    tkuMakeCheckboxes [set fwPrompt$nField] \
			-orientation h \
			-checkboxes [list \
                           [list -type text -label "$aArgs(-prompt$nField)" \
				-variable sFileName$nField]
		       ] 
		}
		default { continue; }
	    }
	    
	    # if they requested a note, make one, otherwise just an empty
	    # frame.
	    set fwNote $wwDialog.fwNote$nField
	    if { [string match "$aArgs(-note$nField)" ""] == 0 } {
		tkuMakeNormalLabel $fwNote \
		    -label "$aArgs(-note$nField)"
	    } else {
		frame $fwNote
	    }
	    
	    # pack this prompt and note
	    pack [set fwPrompt$nField] $fwNote \
		-side top       \
		-expand yes     \
		-fill x         \
		-padx 5         \
		-pady 5
	}
	
	# create the buttons. bind the ok command and the cancel command.
	# before calling the ok command, update the file selector variables.
	# TODO: nicer way to write this command string?
	tkuMakeCancelOKButtons $wwDialog.fwButtons $wwDialog \
	    -okCmd  "catch { tkuUpdateFileSelectorVariable $fwPrompt1 };\
                     catch { tkuUpdateFileSelectorVariable $fwPrompt2 };\
                     catch { tkuUpdateFileSelectorVariable $fwPrompt3 };\
                     catch { tkuUpdateFileSelectorVariable $fwPrompt4 };\
                     catch { tkuUpdateFileSelectorVariable $fwPrompt5 };\
                     $aArgs(-okCmd)"

	pack $wwDialog.fwButtons
	
	# after the next idle, the window will be mapped. set the min
	# width to our width and the min height to the mapped height.
	after idle [format {
	    update idletasks
	    wm minsize %s %d [winfo reqheight %s]
	} $wwDialog $knWidth $wwDialog] 
    }
}

# ============================================================ FILE SELECTORS

# tkuMakeFileSelector
# -variable : variable to set with filename (required)
# -text : text prompt
# -command : command to execute when file is chosen
# -defaultfunc : function to call to get default dir
# -defaultdir : default dir
# -shortcutdirs : dirs to go in shortcut menu
proc tkuMakeFileSelector { ifwTop args } {
    
    # set default arguments for all fields
    set aArgs(-text) "Choose a file:"

    # Set menu items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-variable} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeFileSelector: no $arg specified"
	    return
	}
    }

    frame $ifwTop -width 200
    
    # the entry
    tixLabelEntry $ifwTop.ew \
	-label $aArgs(-text) \
	-labelside acrosstop \
	-options "entry.textVariable $aArgs(-variable) \
                  entry.expand yes \
                  entry.fill x \
                  label.font [tkuLabelFont]"
    
    [$ifwTop.ew subwidget entry] icursor end

    set sDefaultDirOption ""
    if { [info exists aArgs(-defaultfunc)] } {
	set sDefaultDirOption "-defaultfunc $aArgs(-defaultfunc)"
    }
    if { [info exists aArgs(-defaultdir)] } {
	set sDefaultDirOption "-defaultdir $aArgs(-defaultdir)"
    }
    set sShortcutDirsOption ""
    if { [info exists aArgs(-shortcutdirs)] } {
	set sShortcutDirsOption "-shortcutdirs $aArgs(-shortcutdirs)"
    }
    set sCommandOption ""
    if { [info exists aArgs(-command)] } {
	set sCommandOption "-command [list $aArgs(-command)]"
    }

    # the browse button
    button $ifwTop.bw \
	-text "Browse..." \
	-command "tkuBrowseFile -variable $aArgs(-variable) $sDefaultDirOption $sShortcutDirsOption $sCommandOption"
    
    # pack it in a grid
    grid $ifwTop.ew -column 0 -row 0 -sticky ew
    grid $ifwTop.bw -column 1 -row 0
    grid columnconfigure $ifwTop 0 -weight 1
    grid columnconfigure $ifwTop 1 -weight 0
}

# tkuBrowseFile
# -variable : variable name to set (required)
# -command : command to execute when file is chosen
# -defaultfunc : function to call to get default dir
# -defaultdir : default dir
# -shortcutdirs : dirs to go in shortcut menu
proc tkuBrowseFile { args } {
    global gPostHandleSelectFileCommand

    # Set menu items and make sure we have the ones we require,
    set aArgs(-command) ""    
    array set aArgs $args
    foreach arg {-variable} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuBrowseFile: no $arg specified"
	    return
	}
    }

    # create the dialog box if it doesn't already exist
    set wwDirDlog [tix filedialog tixFileSelectDialog]
    
    # set the default location. if it's actually returning a file
    # name, get just the directory portion instead.
    set fnDefaultDir ""
    if { [info exists aArgs(-defaultfunc)] } {
	set fnDefaultDir [$aArgs(-defaultfunc)]
    }
    if { [info exists aArgs(-defaultdir)] } {
	set fnDefaultDir $aArgs(-defaultdir)
    }
    if { $fnDefaultDir != "" } {
	if { [file isfile $fnDefaultDir] } {
	    set fnDefaultDir [file dirname $fnDefaultDir]
	}
	[$wwDirDlog subwidget fsbox] configure -directory $fnDefaultDir
    }

    # add shortcuts
    if { [info exists aArgs(-shortcutdirs)] } {
	foreach sDirectory $aArgs(-shortcutdirs) {
	    [[$wwDirDlog subwidget fsbox] subwidget filter] \
		appendhistory $sDirectory
	}
    }
    
    # when they click ok, call the tkuHandleSelectDirectory function,
    # passing in the variable from the parent dialog.
    $wwDirDlog config -command "tkuHandleSelectFile $aArgs(-variable)"
    
    $wwDirDlog popup

    set gPostHandleSelectFileCommand $aArgs(-command)
}

proc tkuHandleSelectFile { iVariable ifnFile } {
    global gPostHandleSelectFileCommand
    # set the variable.
    upvar $iVariable theVar 
    set theVar $ifnFile
    if { $gPostHandleSelectFileCommand != "" } {
	uplevel #0 {eval $gPostHandleSelectFileCommand}
    }
}

proc tkuUpdateFileSelectorVariable { ifwSelector } {
    $ifwSelector.ew update;
}

# tkuMakeDirectorySelector:
# -variable : variable to set with dirname (required)
# -text : text prompt
# -defaultfunc : function to call to get default dir
# -defaultdir : default dir
# -shortcutdirs : dirs to go in shortcut menu
proc tkuMakeDirectorySelector { ifwTop args } {
    
    # set default arguments for all fields
    set aArgs(-text) "Choose a directory:"

    # Set menu items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-variable} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuMakeDirectorySelector: no $arg specified"
	    return
	}
    }

    frame $ifwTop -width 200
    
    # the entry
    tixLabelEntry $ifwTop.ew \
	-label $aArgs(-text) \
	-labelside acrosstop \
	-options "entry.textVariable $aArgs(-variable) \
                  entry.expand yes \
                  entry.fill x \
                  label.font [tkuLabelFont]"
    
    [$ifwTop.ew subwidget entry] icursor end

    set sDefaultDirOption ""
    if { [info exists aArgs(-defaultfunc)] } {
	set sDefaultDirOption "-defaultfunc $aArgs(-defaultfunc)"
    }
    if { [info exists aArgs(-defaultdir)] } {
	set sDefaultDirOption "-defaultdir $aArgs(-defaultdir)"
    }
    set sShortcutDirsOption ""
    if { [info exists aArgs(-shortcutdirs)] } {
	set sShortcutDirsOption "-shortcutdirs $aArgs(-shortcutdirs)"
    }

    # the browse button
    button $ifwTop.bw \
	-text "Browse..." \
	-command "tkuBrowseDirectory -variable $aArgs(-variable) $sDefaultDirOption $sShortcutDirsOption"
    
    # pack it in a grid
    grid $ifwTop.ew -column 0 -row 0 -sticky ew
    grid $ifwTop.bw -column 1 -row 0
    grid columnconfigure $ifwTop 0 -weight 1
    grid columnconfigure $ifwTop 1 -weight 0
}

# tkuBrowseDirectory
# -variable : variable name to set (required)
# -defaultfunc : function to call to get default dir
# -defaultdir : default dir
# -shortcutdirs : dirs to go in shortcut menu
proc tkuBrowseDirectory { args } {
    
    # Set menu items and make sure we have the ones we require,
    array set aArgs $args
    foreach arg {-variable} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuBrowseDirectory: no $arg specified"
	    return
	}
    }

    # create the dialog box if it doesn't already exist
    set wwDirDlog .wwDirDlog
    if ![winfo exists $wwDirDlog] {
	tixDirSelectDialog $wwDirDlog
    }
    
    # set the default location. if it's actually returning a file
    # name, get just the directory portion instead.
    set fnDefaultDir ""
    if { [info exists aArgs(-defaultfunc)] } {
	set fnDefaultDir [$aArgs(-defaultfunc)]
    }
    if { [info exists aArgs(-defaultdir)] } {
	set fnDefaultDir $aArgs(-defaultdir)
    }
    if { $fnDefaultDir != "" } {
	if { [file isfile $fnDefaultDir] } {
	    set fnDefault [file dirname $fnDefaultDir]
	}
	[$wwDirDlog subwidget dirbox] configure -value $fnDefaultDir
    }
    
    # add shortcuts
    if { [info exists aArgs(-shortcutdirs)] } {
	foreach sDirectory $aArgs(-shortcutdirs) {
	    [[[$wwDirDlog subwidget dirbox] subwidget dircbx] subwidget combo]\
		appendhistory $sDirectory
	}
    }
    
    # when they click ok, call the tkuHandleSelectDirectory function,
    # passing in the variable from the parent dialog.
    $wwDirDlog config -command "tkuHandleSelectDirectory $aArgs(-variable)"
    
    $wwDirDlog popup
}

proc tkuHandleSelectDirectory { iVariable ifnDir } {
    # set the variable.
    upvar $iVariable theVar 
    set theVar $ifnDir
}

proc tkuUpdateDirectorySelectorVariable { ifwSelector } {
    $ifwSelector.ew update;
}


# ============================================================ BUTTON GROUPS

# tkuMakeCancelOKButtons
# -okCmd : command to execute when OK button is pressed
# -cancelCmd : command to execute when Cancel button is pressed
proc tkuMakeCancelOKButtons { ifwTop iwwTop args } {

    global kLabelFont

    # Set menu items and make sure we have the ones we require,
    array set aArgs $args

    set okCmd ""
    if {[info exists aArgs(-okCmd)]} { set okCmd $aArgs(-okCmd) }
    set cancelCmd ""
    if {[info exists aArgs(-cancelCmd)]} { set cancelCmd $aArgs(-cancelCmd) }

    frame $ifwTop
    
    button $ifwTop.bwOK \
      -text "OK" \
      -command "$okCmd; tkuCloseDialog $iwwTop"

    button $ifwTop.bwCancel \
      -text "Cancel" \
      -command "$cancelCmd; tkuCloseDialog $iwwTop"

    bind $iwwTop <Return> \
      "$ifwTop.bwOK flash; $ifwTop.bwOK invoke"
    bind $iwwTop <Escape> \
      "$ifwTop.bwCancel flash; $ifwTop.bwCancel invoke"

    pack $ifwTop.bwOK $ifwTop.bwCancel \
      -side right \
      -padx 5 \
      -pady 5
}

# tkuMakeCloseButton
# -closeCmd : command to execute when Cancel button is pressed
proc tkuMakeCloseButton { ifwTop iwwTop args } {

    global kLabelFont

    # Set menu items and make sure we have the ones we require,
    array set aArgs $args

    set closeCmd ""
    if {[info exists aArgs(-closeCmd)]} { set closeCmd $aArgs(-closeCmd) }

    frame $ifwTop
    
    button $ifwTop.bwClose \
      -text "Close" \
      -command "$closeCmd; tkuCloseDialog $iwwTop"

    bind $iwwTop <Escape> \
      "$ifwTop.bwClose flash; $ifwTop.bwClose invoke"

    pack $ifwTop.bwClose \
      -side right \
      -padx 5 \
      -pady 5
}


# ============================================================ DIALOG BOXES

proc tkuCreateDialog { iwwTop isTitle iArgs } {

    global gDialog
    
    if [winfo exists $iwwTop] {
	switch -- [wm state $iwwTop] {
	    normal {
		raise $iwwTop 
	    }
	    withdrawn - iconic {
		wm deiconify $iwwTop
		catch {
		    wm geometry $iwwTop $gDialog(geo,$iwwTop)
		}
	    }
	}
	return 0
    } else {
	eval {toplevel $iwwTop} $iArgs
	wm title $iwwTop $isTitle
	return 1
    }
}

proc tkuCloseDialog { iwwTop } {
    
    global gDialog
    
    catch {
	set gDialog(geo,$iwwTop) [wm geometry $iwwTop]
	#  wm withdraw $iwwTop
	destroy $iwwTop
    }
}

# tkuCreateMsgDlog
# -msg : message for the dlog (required)
# -variable : alternatively, the variable containing text for the msg dlog
# -parent : parent window on which to center
# -width : width of the dlog
# -height : height of the dlog
proc tkuCreateMsgDlog { args } {
    
    # set default arguments for all fields
    set aArgs(-width) 200
    set aArgs(-height) 50
    set aArgs(-parent) ""
    set aArgs(-variable) ""
    
    array set aArgs $args
    foreach arg {-msg} {
	if {![info exists aArgs($arg)]} {
	    puts "tkuCreateMsgDlog: no $arg specified"
	    return
	}
    }

    if { $aArgs(-variable) != "" } {
	upvar #0 $aArgs(-variable) msgVariable
	puts "msgVariable = $msgVariable"
    }

    set width $aArgs(-width)
    set height $aArgs(-height)
    toplevel .msgDlog -width $width -height $height -borderwidth 0
    
    set x 0
    set y 0
    if { $aArgs(-parent) != "" } {

	set parent $aArgs(-parent)
	set x [expr [winfo x $parent] + ([winfo width $parent] / 2) - \
		($width / 2)]
	set y [expr [winfo y $parent] + ([winfo height $parent] / 2) - \
		($height / 2)]
    }

    frame .msgDlog.fw
    if { $aArgs(-variable) != "" } {
	label .msgDlog.fw.lw -textvariable msgVariable
    } else {
	label .msgDlog.fw.lw -text $aArgs(-msg)
    }

    pack .msgDlog.fw.lw
    pack .msgDlog.fw -expand yes -fill both

    wm geometry .msgDlog ${width}x${height}+${x}+${y}
    update idletasks

    return .msgDlog
}

proc tkuDestroyMsgDlog { iTop } {

    destroy $iTop
}

proc tkuErrorDlog { isMsg } {

    tk_messageBox -type ok \
      -icon error \
      -message $isMsg \
      -title "Error"
}

proc tkuFormattedErrorDlog { isTitle isMsg isDesc } {

    global gDialog

    set wwDialog .wwFormattedErrorDlog

    # try to create the dlog...
    if { [tkuCreateDialog $wwDialog "Error" {-borderwidth 10}] } {
	
	set fwText       $wwDialog.fwText
	set fwButtons    $wwDialog.fwButtons
	
	text $fwText -width 40 \
	    -height 10 \
	    -spacing3 10 \
	    -relief flat \
	    -wrap word
	$fwText insert end "Error: $isTitle \n" {tTitle}
	$fwText insert end "$isMsg \n" {tMsg}
	$fwText insert end "$isDesc \n" {tDesc}
	$fwText tag configure tTitle -font [tkuLabelFont]
	$fwText tag configure tMsg -font [tkuNormalFont]
	$fwText tag configure tDesc -font [tkuNormalFont]
	$fwText configure -state disabled
	
	# button.
	tkuMakeCloseButton $fwButtons $wwDialog
	
	pack $fwText $fwButtons \
	    -side top       \
	    -expand yes     \
	    -fill x         \
	    -padx 5         \
	    -pady 5
    }
}

proc tkuAlertDlog { isMsg } {

    tk_messageBox -type ok \
      -icon info \
      -message $isMsg \
      -title "Note"
}

